@@ -1,5 +1,5 @@
-/* $Id: error.c,v 1.26 2005/03/18 20:21:55 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: error.c,v 1.28 2006/05/23 05:56:49 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* the logging subsystem */
@@ -12,6 +12,7 @@
 #if HAVE_SYSLOG_H
 #include <syslog.h>
 #endif
+#include <sys/utsname.h>
 #include <time.h>	/* time_t, strftime() */
 #include <sys/time.h>	/* gettimeofday() */
 #include <stdio.h>
@@ -44,6 +45,8 @@ struct diag_opts {
    int logfacility;
    bool micros;
    int exitstatus;	/* pass signal number to error exit */
+   bool withhostname;	/* in custom logs add hostname */
+   char *hostname;
 } ;
 
 
@@ -123,6 +126,13 @@ void diag_set_int(char what, int arg) {
    case 'D': diagopts.msglevel = arg; break;
    case 'e': diagopts.exitlevel = arg; break;
    case 'x': diagopts.exitstatus = arg; break;
+   case 'h': diagopts.withhostname = arg;
+      if ((diagopts.hostname = getenv("HOSTNAME")) == NULL) {
+	 struct utsname ubuf;
+	 uname(&ubuf);
+	 diagopts.hostname = strdup(ubuf.nodename);
+      }
+      break;
    default: msg(E_ERROR, "unknown diagnostic option %c", what);
    }
 }
@@ -175,7 +185,7 @@ void msg(int level, const char *format, ...) {
 #if HAVE_STRFTIME
       if (diagopts.micros) {
 	 bytes = strftime(buff, 20, "%Y/%m/%d %H:%M:%S", localtime(&nowt));
-	 bytes += sprintf(buff+19, ".%06lu ", now.tv_usec);
+	 bytes += sprintf(buff+19, "."F_tv_usec" ", now.tv_usec);
       } else {
 	 bytes =
 	    strftime(buff, 21, "%Y/%m/%d %H:%M:%S ", localtime(&nowt));
@@ -202,6 +212,9 @@ void msg(int level, const char *format, ...) {
    }
 #endif /* !HAVE_GETTIMEOFDAY */
    bufp = buff + bytes;
+   if (diagopts.withhostname) {
+      bytes = sprintf(bufp, "%s ", diagopts.hostname), bufp+=bytes;
+   }
    bytes = sprintf(bufp, "%s["F_pid"] ", diagopts.progname, getpid());
    bufp += bytes;
    syslp = bufp;
