@@ -1,5 +1,5 @@
-/* $Id: filan.c,v 1.36 2004/12/03 22:40:07 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: filan.c,v 1.40 2006/07/13 06:32:57 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* the subroutine filan makes a "FILe descriptor ANalysis". It checks the
@@ -43,6 +43,7 @@ static int printtime(FILE *outfile, time_t time);
 
 static int headprinted;
 
+/* analyse a file system entry, referred by file name */
 int filan_file(const char *filename, FILE *outfile) {
    int fd = -1;
    int result;
@@ -59,7 +60,7 @@ int filan_file(const char *filename, FILE *outfile) {
       result = Stat(filename, &buf);
 #endif /* !HAVE_STAT64 */
       if (result < 0) {
-	 Error3("stat(\"%s\", %p): %s", filename, &buf, strerror(errno));
+	 Warn3("stat(\"%s\", %p): %s", filename, &buf, strerror(errno));
       }
    } else {
 #if HAVE_STAT64
@@ -68,7 +69,7 @@ int filan_file(const char *filename, FILE *outfile) {
       result = Lstat(filename, &buf);
 #endif /* !HAVE_STAT64 */
       if (result < 0) {
-	 Error3("lstat(\"%s\", %p): %s", filename, &buf, strerror(errno));
+	 Warn3("lstat(\"%s\", %p): %s", filename, &buf, strerror(errno));
       }
    }
    switch (buf.st_mode&S_IFMT) {
@@ -89,9 +90,12 @@ int filan_file(const char *filename, FILE *outfile) {
       }
    }
      
-   return filan_stat(&buf, fd, -1, outfile);
+   result = filan_stat(&buf, fd, -1, outfile);
+   fputc('\n', outfile);
+   return result;
 }
 
+/* analyze a file descriptor */
 int filan_fd(int fd, FILE *outfile) {
 #if HAVE_STAT64
    struct stat64 buf = {0};
@@ -110,13 +114,84 @@ int filan_fd(int fd, FILE *outfile) {
       if (errno == EBADF) {
          Debug2("fstat(%d): %s", fd, strerror(errno));
       } else {
-         Error2("fstat(%d): %s", fd, strerror(errno));
+         Warn2("fstat(%d): %s", fd, strerror(errno));
       }
       return -1;
    }
    Debug2("fd %d is a %s", fd, getfiletypestring(buf.st_mode));
 
-   return filan_stat(&buf, fd, fd, outfile);
+   result = filan_stat(&buf, fd, fd, outfile);
+
+   if (result >= 0) {
+      /* even more dynamic info */
+      { /* see if data is available */
+	 struct pollfd ufds;
+	 ufds.fd = fd;
+	 ufds.events = POLLIN|POLLPRI|POLLOUT|POLLRDNORM|POLLRDBAND|POLLWRNORM|POLLWRBAND
+#ifdef POLLMSG
+	    |POLLMSG
+#endif
+	    ;
+	 if (Poll(&ufds, 1, 0) < 0) {
+	    Warn4("poll({%d, %hd, %hd}, 1, 0): %s",
+		   ufds.fd, ufds.events, ufds.revents, strerror(errno));
+	 } else {
+	    fputs("poll: ", outfile);
+	    if (ufds.revents & POLLIN)   fputs("IN,", outfile);
+	    if (ufds.revents & POLLPRI)  fputs("PRI,", outfile);
+	    if (ufds.revents & POLLOUT)  fputs("OUT,", outfile);
+	    if (ufds.revents & POLLERR)  fputs("ERR,", outfile);
+	    if (ufds.revents & POLLNVAL) fputs("NVAL,", outfile);
+#ifdef FIONREAD
+	    if (ufds.revents & POLLIN) {
+	       size_t sizet;
+	       if ((result = Ioctl(fd, FIONREAD, &sizet) >= 0)) {
+		  fprintf (outfile, "; FIONREAD="F_Zu, sizet);
+	       }
+	    }
+#endif /* defined(FIONREAD) */
+#if WITH_SOCKET
+	    if (isasocket(fd)) {
+	       char _peername[SOCKADDR_MAX];
+	       struct sockaddr *pa = (struct sockaddr *)_peername;
+	       struct msghdr msgh = {0};
+	       char peekbuff[0];
+#if HAVE_STRUCT_IOVEC
+	       struct iovec iovec;
+#endif
+	       char ctrlbuff[5120];
+	       ssize_t bytes;
+
+	       fputs("; ", outfile);
+	       msgh.msg_name = pa;
+	       msgh.msg_namelen = sizeof(*pa);
+#if HAVE_STRUCT_IOVEC
+	       iovec.iov_base = peekbuff;
+	       iovec.iov_len  = sizeof(peekbuff);
+	       msgh.msg_iov = &iovec;
+	       msgh.msg_iovlen = 1;
+#endif
+#if HAVE_STRUCT_MSGHDR_MSGCONTROL
+	       msgh.msg_control = ctrlbuff;
+#endif
+#if HAVE_STRUCT_MSGHDR_MSGCONTROLLEN
+	       msgh.msg_controllen = sizeof(ctrlbuff);
+#endif
+#if HAVE_STRUCT_MSGHDR_MSGFLAGS
+	       msgh.msg_flags = 0;
+#endif
+	       if ((bytes = Recvmsg(fd, &msgh, MSG_PEEK)) < 0) {
+		  Warn1("recvmsg(): %s", strerror(errno));
+	       } else {
+		  fprintf(outfile, "recvmsg="F_Zd", ", bytes);
+	       }
+	    }
+#endif /* WITH_SOCKET */
+	 }	 
+      }
+   }
+   fputc('\n', outfile);
+   return 0;
 }
 
 
@@ -299,7 +374,7 @@ int filan_stat(
 #if WITH_SOCKET
       result = sockan(statfd, outfile);
 #else
-      Error("SOCKET support not compiled in");
+      Warn("SOCKET support not compiled in");
       return -1;
 #endif /* !WITH_SOCKET */
       break;
@@ -307,7 +382,6 @@ int filan_stat(
    }
   }
    /* ioctl() */
-   fputc('\n', outfile);
    return 0;
 }
 
@@ -339,7 +413,7 @@ int cdevan(int fd, FILE *outfile) {
    int ret;
 
    if ((ret = Isatty(fd)) < 0) {
-      Error2("isatty(%d): %s", fd, strerror(errno));
+      Warn2("isatty(%d): %s", fd, strerror(errno));
       return -1;
    }
    if (ret > 0) {
@@ -348,13 +422,13 @@ int cdevan(int fd, FILE *outfile) {
       int i;
 
       if ((name = Ttyname(fd)) == NULL) {
-	 /*Error2("ttyname(%d): %s", fd, strerror(errno));*/
+	 /*Warn2("ttyname(%d): %s", fd, strerror(errno));*/
 	 fputs("\tNULL", outfile);
       } else {
 	 fprintf(outfile, "\t%s", name);
       }
       if (Tcgetattr(fd, &termarg) < 0) {
-	 Error3("tcgetattr(%d, %p): %s", fd, &termarg, strerror(errno));
+	 Warn3("tcgetattr(%d, %p): %s", fd, &termarg, strerror(errno));
 	 return -1;
       }
       fprintf(outfile, " \tIFLAGS=%08x OFLAGS=%08x CFLAGS=%08x LFLAGS=%08x",
@@ -506,7 +580,7 @@ int sockan(int fd, FILE *outfile) {
    result = Getsockname(fd, (struct sockaddr *)&sockname, &namelen);
    if (result < 0) {
       putc('\n', outfile);
-      Error2("getsockname(%d): %s", fd, strerror(errno));
+      Warn2("getsockname(%d): %s", fd, strerror(errno));
       return -1;
    }
    fputc('\t', outfile);
@@ -517,16 +591,18 @@ int sockan(int fd, FILE *outfile) {
    result = Getpeername(fd, (struct sockaddr *)&peername, &namelen);
    if (result < 0) {
       putc('\n', outfile);
-      Error2("getpeername(%d): %s", fd, strerror(errno));
+      Warn2("getpeername(%d): %s", fd, strerror(errno));
+   } else {
+      /* only valid if getpeername() succeeded */
+      fputs(" <-> ", outfile);
+      fprintf(outfile, "%s\t",
+	      sockaddr_info((struct sockaddr *)&peername,
+			    nambuff, sizeof(nambuff)));
    }
-   fputs(" <-> ", outfile);
-   fprintf(outfile, "%s\t",
-	   sockaddr_info((struct sockaddr *)&peername,
-			 nambuff, sizeof(nambuff)));
 
 #if 0 && defined(SIOCGIFNAME)
    if ((result = Ioctl(fd, SIOCGIFNAME, &ifc)) < 0) {
-      Error3("ioctl(%d, SIOCGIFNAME, %p): %s", fd, &ifc, strerror(errno));
+      Warn3("ioctl(%d, SIOCGIFNAME, %p): %s", fd, &ifc, strerror(errno));
    } else {
       fprintf(outfile, "IFNAME=\"%s\"\t", ifc.ifr_name);
    }
