@@ -1,5 +1,5 @@
-/* $Id: socat.c,v 1.94 2005/09/04 09:39:51 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: socat.c,v 1.106 2006/07/13 21:16:04 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the main source, including command line option parsing, general
@@ -28,8 +28,9 @@ struct {
    size_t bufsiz;
    bool verbose;
    bool verbhex;
-   unsigned int timeout;	/* with ignoreeof, reread after seconds */
+   struct timeval pollintv;	/* with ignoreeof, reread after seconds */
    struct timeval closwait;	/* after close of x, die after seconds */
+   struct timeval total_timeout;/* when nothing happens, die after seconds */
    bool debug;
    bool strictopts;	/* stop on errors in address options */
    char logopt;		/* y..syslog; s..stderr; f..file; m..mixed */
@@ -40,8 +41,9 @@ struct {
    8192,	/* bufsiz */
    false,	/* verbose */
    false,	/* verbhex */
-   0,		/* timeout */
+   {1,0},	/* pollintv */
    {0,500000},	/* closwait */
+   {0,0},	/* total_timeout */
    0,		/* debug */
    0,		/* strictopts */
    's',		/* logopt */
@@ -63,7 +65,7 @@ void crlftolf(char **in, ssize_t *len, size_t bufsiz);
 
 static int socat_lock(void);
 static void socat_unlock(void);
-static int socat_newchild(pid_t pid);
+static int socat_newchild(void);
 
 static const char socatversion[] =
 #include "./VERSION"
@@ -88,6 +90,13 @@ int main(int argc, const char *argv[]) {
    int lockrc;
 
    diag_set('p', strchr(argv[0], '/') ? strrchr(argv[0], '/')+1 : argv[0]);
+
+   /* we must init before applying options because env settings have lower
+      priority and are to be overridden by options */
+   if (xioinitialize() != 0) {
+      Exit(1);
+   }
+
    xiosetopt('p', "!!");
    xiosetopt('o', ":");
 
@@ -109,25 +118,35 @@ int main(int argc, const char *argv[]) {
 #endif
       case 'l':
 	 switch (arg1[0][2]) {
-	 case 'm':
-	    /* mixed mode: stderr, then switch to syslog */
+	 case 'm': /* mixed mode: stderr, then switch to syslog; + facility */
 	    diag_set('s', NULL);
 	    xiosetopt('l', "m");
 	    socat_opts.logopt = arg1[0][2];
 	    xiosetopt('y', &arg1[0][3]);
 	    break;
-	 case 'y':
-	 case 'f':
-	 case 's':
-	    /* >stderr, s>yslog, >file */
+	 case 'y': /* syslog + facility */
 	    diag_set(arg1[0][2], &arg1[0][3]);
 	    break;
-	 case 'p':
-	    diag_set('p', &arg1[0][3]);
+	 case 'f': /* to file, +filename */
+	 case 'p': /* artificial program name */
+	    if (arg1[0][3]) {
+	       diag_set(arg1[0][2], &arg1[0][3]);
+	    } else if (arg1[1]) {
+	       diag_set(arg1[0][2], arg1[1]);
+	       ++arg1, --argc;
+	    } else {
+	       Error1("option -l%c requires an argument; use option \"-h\" for help", arg1[0][2]);
+	    }
+	    break;
+	 case 's': /* stderr */
+	    diag_set(arg1[0][2], NULL);
 	    break;
 	 case 'u':
 	    diag_set('u', NULL);
 	    break;
+	 case 'h':
+	    diag_set_int('h', true);
+	    break;
 	 default:
 	    Error1("unknown log option \"%s\"; use option \"-h\" for help", arg1[0]);
 	    break;
@@ -141,7 +160,7 @@ int main(int argc, const char *argv[]) {
 	    ++arg1, --argc;
 	    if ((a = *arg1) == NULL) {
 	       Error("option -b requires an argument; use option \"-h\" for help");
-	       socat_usage(stderr); Exit(1);
+	       Exit(1);
 	    }
 	 }
 	 socat_opts.bufsiz = strtoul(a, (char **)&a, 0);
@@ -154,7 +173,7 @@ int main(int argc, const char *argv[]) {
 	    ++arg1, --argc;
 	    if ((a = *arg1) == NULL) {
 	       Error("option -t requires an argument; use option \"-h\" for help");
-	       socat_usage(stderr); Exit(1);
+	       Exit(1);
 	    }
 	 }
 	 rto = strtod(a, (char **)&a);
@@ -162,6 +181,20 @@ int main(int argc, const char *argv[]) {
 	 socat_opts.closwait.tv_usec =
 	    (rto-socat_opts.closwait.tv_sec) * 1000000; 
 	 break;
+      case 'T':  if (arg1[0][2]) {
+	    a = *arg1+2;
+	 } else {
+	    ++arg1, --argc;
+	    if ((a = *arg1) == NULL) {
+	       Error("option -T requires an argument; use option \"-h\" for help");
+	       Exit(1);
+	    }
+	 }
+	 rto = strtod(a, (char **)&a);
+	 socat_opts.total_timeout.tv_sec = rto;
+	 socat_opts.total_timeout.tv_usec =
+	    (rto-socat_opts.total_timeout.tv_sec) * 1000000; 
+	 break;
       case 'u': socat_opts.lefttoright = true; break;
       case 'U': socat_opts.righttoleft = true; break;
       case 'g': xioopts_ignoregroups = true; break;
@@ -173,7 +206,7 @@ int main(int argc, const char *argv[]) {
 	    ++arg1, --argc;
 	    if ((socat_opts.lock.lockfile = *arg1) == NULL) {
 	       Error("option -L requires an argument; use option \"-h\" for help");
-	       socat_usage(stderr); Exit(1);
+	       Exit(1);
 	    }
 	 }
 	 break;
@@ -185,13 +218,24 @@ int main(int argc, const char *argv[]) {
 	    ++arg1, --argc;
 	    if ((socat_opts.lock.lockfile = *arg1) == NULL) {
 	       Error("option -W requires an argument; use option \"-h\" for help");
-	       socat_usage(stderr); Exit(1);
+	       Exit(1);
 	    }
 	 }
 	 socat_opts.lock.waitlock = true;
 	 socat_opts.lock.intervall.tv_sec  = 1;
 	 socat_opts.lock.intervall.tv_nsec = 0;
 	 break;
+#if WITH_IP4 || WITH_IP6
+#if WITH_IP4
+      case '4':
+#endif
+#if WITH_IP6
+      case '6':
+#endif
+	 xioopts.default_ip = arg1[0][1];
+	 xioopts.preferred_ip = arg1[0][1];
+	 break;
+#endif /* WITH_IP4 || WITH_IP6 */
       case '\0':
       case ',':
       case ':': break;	/* this "-" is a variation of STDIO */
@@ -201,7 +245,7 @@ int main(int argc, const char *argv[]) {
 	    break;
 	 }
 	 Error1("unknown option \"%s\"; use option \"-h\" for help", arg1[0]);
-	 socat_usage(stderr); Exit(1);
+	 Exit(1);
       }
       /* the leading "-" might be a form of the first address */
       xioinqopt('p', buff, sizeof(buff));
@@ -213,7 +257,6 @@ int main(int argc, const char *argv[]) {
    }
    if (argc != 2) {
       Error1("exactly 2 addresses required (there are %d); use option \"-h\" for help", argc);
-      /*0 socat_usage(stderr);*/
       Exit(1);
    }
    if (socat_opts.lefttoright && socat_opts.righttoleft) {
@@ -289,16 +332,24 @@ void socat_usage(FILE *fd) {
    fputs("      -lm[facility]  mixed log mode (stderr during initialization, then syslog)\n", fd);
    fputs("      -lp<progname>  set the program name used for logging\n", fd);
    fputs("      -lu            use microseconds for logging timestamps\n", fd);
+   fputs("      -lh            add hostname to log messages\n", fd);
    fputs("      -v     verbose data traffic, text\n", fd);
    fputs("      -x     verbose data traffic, hexadecimal\n", fd);
    fputs("      -b<size_t>     set data buffer size (8192)\n", fd);
    fputs("      -s     sloppy (continue on error)\n", fd);
    fputs("      -t<timeout>    wait seconds before closing second channel\n", fd);
+   fputs("      -T<timeout>    total inactivity timeout in seconds\n", fd);
    fputs("      -u     unidirectional mode (left to right)\n", fd);
    fputs("      -U     unidirectional mode (right to left)\n", fd);
    fputs("      -g     do not check option groups\n", fd);
    fputs("      -L <lockfile>  try to obtain lock, or fail\n", fd);
    fputs("      -W <lockfile>  try to obtain lock, or wait\n", fd);
+#if WITH_IP4
+   fputs("      -4     prefer IPv4 if version is not explicitly specified\n", fd);
+#endif
+#if WITH_IP6
+   fputs("      -6     prefer IPv6 if version is not explicitly specified\n", fd);
+#endif
 }
 
 
@@ -421,6 +472,11 @@ void socat_version(FILE *fd) {
 #else
    fputs("  #undef WITH_OPENSSL\n", fd);
 #endif
+#ifdef WITH_FIPS
+   fprintf(fd, "  #define WITH_FIPS %d\n", WITH_FIPS);
+#else
+   fputs("  #undef WITH_FIPS\n", fd);
+#endif
 #ifdef WITH_LIBWRAP
    fprintf(fd, "  #define WITH_LIBWRAP %d\n", WITH_LIBWRAP);
 #else
@@ -617,9 +673,8 @@ int childleftdata(xiofile_t *xfd) {
 }
 
 int xiotransfer(xiofile_t *inpipe, xiofile_t *outpipe,
-		unsigned char **buff, size_t bufsiz, const char *prefix);
+		unsigned char **buff, size_t bufsiz, bool righttoleft);
 
-int sleepon = 0;	/* if last select was not active, then sleep so long */
 bool mayrd1;		/* sock1 has read data or eof, according to select() */
 bool mayrd2;		/* sock2 has read data or eof, according to select() */
 bool maywr1;		/* sock1 can be written to, according to select() */
@@ -630,11 +685,12 @@ bool maywr2;		/* sock2 can be written to, according to select() */
    returns -1 on error or 0 on success */
 int _socat(void) {
    fd_set in, out, expt;
-   struct timeval timeout, *to;
    int retval;
    unsigned char *buff;
    ssize_t bytes1, bytes2;
+   int polling = 0;	/* handling ignoreeof */
    int wasaction = 1;	/* last select was active, do NOT sleep before next */
+   struct timeval total_timeout;	/* the actual total timeout timer */
 
 #if WITH_FILAN
    if (socat_opts.debug) {
@@ -665,53 +721,95 @@ int _socat(void) {
    }
 #endif /* WITH_FILAN */
 
+   /* when converting nl to crnl, size might double */
    buff = Malloc(2*socat_opts.bufsiz+1);
    if (buff == NULL)  return -1;
 
-   if (socat_opts.timeout) {
-      /*! timeout = socat_opts.timeout;*/	/*!?*/
-      to = &timeout;
-   } else {
-      to = NULL;
-   }
-
    if (socat_opts.logopt == 'm' && xioinqopt('l', NULL, 0) == 'm') {
       Info("switching to syslog");
       diag_set('y', xioopts.syslogfac);
       xiosetopt('l', "\0");
    }
+   total_timeout = socat_opts.total_timeout;
+
    Notice4("starting data transfer loop with FDs [%d,%d] and [%d,%d]",
 	   XIO_GETRDFD(sock1), XIO_GETWRFD(sock1),
 	   XIO_GETRDFD(sock2), XIO_GETWRFD(sock2));
    while (XIO_RDSTREAM(sock1)->eof <= 1 ||
 	  XIO_RDSTREAM(sock2)->eof <= 1) {
-      Debug5("data loop: sock1->eof=%d, sock2->eof=%d, closing=%d, sleepon=%d, wasaction=%d",
-	     XIO_RDSTREAM(sock1)->eof, XIO_RDSTREAM(sock2)->eof, closing, sleepon, wasaction);
+      struct timeval timeout, *to = NULL;
 
-      if (!wasaction) {
-	 Sleep(sleepon);
+      Debug6("data loop: sock1->eof=%d, sock2->eof=%d, closing=%d, wasaction=%d, total_to={"F_tv_sec"."F_tv_usec"}",
+	     XIO_RDSTREAM(sock1)->eof, XIO_RDSTREAM(sock2)->eof,
+	     closing, wasaction,
+	     total_timeout.tv_sec, total_timeout.tv_usec);
+
+      /* for ignoreeof */
+      if (polling) {
+	 if (!wasaction) {
+	    /* yes we could do it with select but I like readable trace output */
+	    if (socat_opts.pollintv.tv_sec)  Sleep(socat_opts.pollintv.tv_sec);
+	    if (socat_opts.pollintv.tv_usec) Usleep(socat_opts.pollintv.tv_usec);
+
+	    if (socat_opts.total_timeout.tv_sec != 0 ||
+		socat_opts.total_timeout.tv_usec != 0) {
+	       if (total_timeout.tv_usec < socat_opts.pollintv.tv_usec) {
+		  total_timeout.tv_usec += 1000000;
+		  total_timeout.tv_sec  -= 1;
+	       }
+	       total_timeout.tv_sec  -= socat_opts.pollintv.tv_sec;
+	       total_timeout.tv_usec -= socat_opts.pollintv.tv_usec;
+	       if (total_timeout.tv_sec < 0 ||
+		   total_timeout.tv_sec == 0 && total_timeout.tv_usec < 0) {
+		  Notice("inactivity timeout triggered");
+		  return 0;
+	       }
+	    }
+
+	 } else {
+	    wasaction = 0;
+	 }
+      }
+
+      if (polling) {
+	 /* there is a ignoreeof poll timeout, use it */
+	 timeout = socat_opts.pollintv;
+	 to = &timeout;
+      } else if (socat_opts.total_timeout.tv_sec != 0 ||
+		 socat_opts.total_timeout.tv_usec != 0) {
+	 /* there might occur a total inactivity timeout */
+	 timeout = socat_opts.total_timeout;
+	 to = &timeout;
+      } else {
+	 to = NULL;
+      }
+
+      if (closing>=1) {
+	 /* first eof already occurred, start end timer */
+	 timeout = socat_opts.closwait;
+	 to = &timeout;
+	 closing = 2;
       }
-      wasaction = 0;
 
-#if 0
-      if (sock1->fd[0].fd >= 0 &&
-	  !(/*sock1->fd[0].ignoreeof &&*/ XIO_RDSTREAM(sock1)->eof)) {
-	 if (XIO_READABLE(sock1))
-	    FD_SET(sock1->fd[0].fd, &in);
-	 FD_SET(sock1->fd[0].fd, &expt); }
-#endif
       do {
+	 int _errno;
 	 FD_ZERO(&in); FD_ZERO(&out); FD_ZERO(&expt);
 
 	 childleftdata(sock1);
 	 childleftdata(sock2);
 
+	 if (closing>=1) {
+	    /* first eof already occurred, start end timer */
+	    timeout = socat_opts.closwait;
+	    to = &timeout;
+	    closing = 2;
+	 }
+
 	 if (XIO_READABLE(sock1) &&
 	     !(XIO_RDSTREAM(sock1)->eof > 1 && !XIO_RDSTREAM(sock1)->ignoreeof) &&
 	     !socat_opts.righttoleft) {
 	    if (!mayrd1) {
 	       FD_SET(XIO_GETRDFD(sock1), &in);
-	       /*0 FD_SET(XIO_GETRDFD(sock1), &expt);*/
 	    }
 	    if (!maywr2) {
 	       FD_SET(XIO_GETWRFD(sock2), &out);
@@ -722,29 +820,22 @@ int _socat(void) {
 	     !socat_opts.lefttoright) {
 	    if (!mayrd2) {
 	       FD_SET(XIO_GETRDFD(sock2), &in);
-	       /*0 FD_SET(XIO_GETRDFD(sock2), &expt);*/
 	    }
 	    if (!maywr1) {
 	       FD_SET(XIO_GETWRFD(sock1), &out);
 	    }
 	 }
-
-	 if (!closing && to) {
-	    to->tv_sec  = socat_opts.timeout;
-	    to->tv_usec = 0;
-	 } else if (closing==1) {
-	    /* first eof just occurred, start end timer */
-	    to = &timeout;
-	    timeout = socat_opts.closwait;
-	    closing = 2;
-	 }
 	 retval = Select(FOPEN_MAX, &in, &out, &expt, to);
+	 _errno = errno;
+	 if (retval < 0 && errno == EINTR) {
+	    Info1("select(): %s", strerror(errno));
+	 }
+	 errno = _errno;
       } while (retval < 0 && errno == EINTR);
 
       /* attention:
 	 when an exec'd process sends data and terminates, it is unpredictable
 	 whether the data or the sigchild arrives first.
-	 
 	 */
 
       if (retval < 0) {
@@ -761,8 +852,19 @@ int _socat(void) {
 #endif
 	    return -1;
       } else if (retval == 0) {
-	 Info1("select timed out (no data within %d seconds)",
-	       socat_opts.timeout); /*! wrong source for value */
+	 Info2("select timed out (no data within %d.%06u seconds)",
+	       closing>=1?socat_opts.closwait.tv_sec:socat_opts.total_timeout.tv_sec,
+	       closing>=1?socat_opts.closwait.tv_usec:socat_opts.total_timeout.tv_usec);
+	 if (polling && !wasaction) {
+	    /* there was a ignoreeof poll timeout, use it */
+	    ;
+	 } else if (socat_opts.total_timeout.tv_sec != 0 ||
+		    socat_opts.total_timeout.tv_usec != 0) {
+	    /* there was a total inactivity timeout */
+	    Notice("inactivity timeout triggered");
+	    return 0;
+	 }
+
 	 if (closing) {
 	    break;
 	 }
@@ -787,8 +889,8 @@ int _socat(void) {
       }
 
       if (mayrd1 && maywr2) {
-	 mayrd1 = maywr2 = false;
-	 if ((bytes1 = xiotransfer(sock1, sock2, &buff, socat_opts.bufsiz, "> "))
+	 mayrd1 = false;
+	 if ((bytes1 = xiotransfer(sock1, sock2, &buff, socat_opts.bufsiz, false))
 	     < 0) {
 	    if (errno != EAGAIN) {
 	       closing = MAX(closing, 1);
@@ -798,6 +900,8 @@ int _socat(void) {
 	       }
 	    }
 	 } else if (bytes1 > 0) {
+	    maywr2 = false;
+	    total_timeout = socat_opts.total_timeout;
 	    wasaction = 1;
 	 }
 	 /* (bytes1 == 0)  handled later */
@@ -806,8 +910,8 @@ int _socat(void) {
       }
 
       if (mayrd2 && maywr1) {
-	 mayrd2 = maywr1 = false;
-	 if ((bytes2 = xiotransfer(sock2, sock1, &buff, socat_opts.bufsiz, "< "))
+	 mayrd2 = false;
+	 if ((bytes2 = xiotransfer(sock2, sock1, &buff, socat_opts.bufsiz, true))
 	     < 0) {
 	    if (errno != EAGAIN) {
 	       closing = MAX(closing, 1);
@@ -817,6 +921,8 @@ int _socat(void) {
 	       }
 	    }
 	 } else if (bytes2 > 0) {
+	    maywr1 = false;
+	    total_timeout = socat_opts.total_timeout;
 	    wasaction = 1;
 	 }
 	 /* (bytes2 == 0)  handled later */
@@ -826,28 +932,28 @@ int _socat(void) {
 
       /* NOW handle EOFs */
 
-      if (bytes1 == 0) {
+      if (bytes1 == 0 || XIO_RDSTREAM(sock1)->eof >= 2) {
 	 if (XIO_RDSTREAM(sock1)->ignoreeof && !closing) {
 	    Debug1("socket 1 (fd %d) is at EOF, ignoring",
 		   XIO_RDSTREAM(sock1)->fd);	/*! */
-	    sleepon = 1;
+	    polling = 1;
 	 } else {
 	    Notice1("socket 1 (fd %d) is at EOF", XIO_GETRDFD(sock1));
-	    xioshutdown(sock2, 1);
+	    xioshutdown(sock2, SHUT_WR);
 	    if (socat_opts.lefttoright) {
 	       break;
 	    }
 	 }
       }
 
-      if (bytes2 == 0) {
+      if (bytes2 == 0 || XIO_RDSTREAM(sock2)->eof >= 2) {
 	 if (XIO_RDSTREAM(sock2)->ignoreeof && !closing) {
 	    Debug1("socket 2 (fd %d) is at EOF, ignoring",
 		   XIO_RDSTREAM(sock2)->fd);
-	    sleepon = 1;
+	    polling = 1;
 	 } else {
 	    Notice1("socket 2 (fd %d) is at EOF", XIO_GETRDFD(sock2));
-	    xioshutdown(sock1, 1);
+	    xioshutdown(sock1, SHUT_WR);
 	    if (socat_opts.righttoleft) {
 	       break;
 	    }
@@ -863,6 +969,76 @@ int _socat(void) {
 }
 
 
+#define MAXTIMESTAMPLEN 128
+/* prints the timestamp to the buffer and terminates it with '\0'. This buffer
+   should be at least MAXTIMESTAMPLEN bytes long.
+   returns 0 on success or -1 if an error occurred */
+int gettimestamp(char *timestamp) {
+   size_t bytes;
+#if HAVE_GETTIMEOFDAY || 1
+   struct timeval now;
+   int result;
+   time_t nowt;
+#else /* !HAVE_GETTIMEOFDAY */
+   time_t now;
+#endif /* !HAVE_GETTIMEOFDAY */
+
+#if HAVE_GETTIMEOFDAY || 1
+   result = gettimeofday(&now, NULL);
+   if (result < 0) {
+      return result;
+   } else {
+      nowt = now.tv_sec;
+#if HAVE_STRFTIME
+      bytes = strftime(timestamp, 20, "%Y/%m/%d %H:%M:%S", localtime(&nowt));
+      bytes += sprintf(timestamp+19, "."F_tv_usec" ", now.tv_usec);
+#else
+      strcpy(timestamp, ctime(&nowt));
+      bytes = strlen(timestamp);
+#endif
+   }
+#else /* !HAVE_GETTIMEOFDAY */
+   now = time(NULL);  if (now == (time_t)-1) {
+      return -1;
+   } else {
+#if HAVE_STRFTIME
+      bytes = strftime(timestamp, 21, "%Y/%m/%d %H:%M:%S ", localtime(&now));
+#else
+      strcpy(timestamp, ctime(&now));
+      bytes = strlen(timestamp);
+#endif
+   }
+#endif /* !HAVE_GETTIMEOFDAY */
+   return 0;
+}
+
+static const char *prefixltor = "> ";
+static const char *prefixrtol = "< ";
+static unsigned long numltor;
+static unsigned long numrtol;
+/* print block header (during verbose or hex dump)
+   returns 0 on success or -1 if an error occurred */
+static int
+   xioprintblockheader(FILE *file, size_t bytes, bool righttoleft) {
+   char timestamp[MAXTIMESTAMPLEN];
+   char buff[128+MAXTIMESTAMPLEN];
+   if (gettimestamp(timestamp) < 0) {
+      return -1;
+   }
+   if (righttoleft) {
+      numrtol+=bytes;
+      sprintf(buff, "%s%s length="F_Zu" from=%lu to=%lu\n",
+	      prefixrtol, timestamp, bytes, numrtol, numrtol-1);
+   } else {
+      numltor+=bytes;
+      sprintf(buff, "%s%s length="F_Zu" from=%lu to=%lu\n",
+	      prefixltor, timestamp, bytes, numltor, numltor-1);
+   }
+   fputs(buff, file);
+   return 0;
+}
+
+
 /* inpipe is suspected to have read data available; read at most bufsiz bytes
    and transfer them to outpipe. Perform required data conversions.
    buff should be at least twice as large as bufsiz, to allow all standard
@@ -874,13 +1050,14 @@ int _socat(void) {
    */
 /* inpipe, outpipe must be single descriptors (not dual!) */
 int xiotransfer(xiofile_t *inpipe, xiofile_t *outpipe,
-		unsigned char **buff, size_t bufsiz, const char *prefix) {
+		unsigned char **buff, size_t bufsiz, bool righttoleft) {
    ssize_t bytes, writt;
 
 	 bytes = xioread(inpipe, *buff, socat_opts.bufsiz);
 	 if (bytes < 0) {
-	    XIO_RDSTREAM(inpipe)->eof = 2;
-	    /*xioshutdown(inpipe, 0);*/
+	    if (errno != EAGAIN)
+	       XIO_RDSTREAM(inpipe)->eof = 2;
+	    /*xioshutdown(inpipe, SHUT_RD);*/
 	    return -1;
 	 }
 	 if (bytes == 0 && XIO_RDSTREAM(inpipe)->ignoreeof && !closing) {
@@ -911,8 +1088,9 @@ int xiotransfer(xiofile_t *inpipe, xiofile_t *outpipe,
 	       const unsigned char *end, *s, *t;
 	       s = *buff;
 	       end = (*buff)+bytes;
+	       xioprintblockheader(stderr, bytes, righttoleft);
 	       while (s < end) {
-		  fputs(prefix, stderr);
+		  /*! prefix? */
 		  j = Min(N, (size_t)(end-s));
 
 		  /* print hex */
@@ -953,10 +1131,11 @@ int xiotransfer(xiofile_t *inpipe, xiofile_t *outpipe,
 	       fputs("--\n", stderr);
 	    } else if (socat_opts.verbose) {
 	       size_t i = 0;
-	       fputs(prefix, stderr);
+	       xioprintblockheader(stderr, bytes, righttoleft);
 	       while (i < (size_t)bytes) {
 		  int c = (*buff)[i];
-		  if (i > 0 && (*buff)[i-1] == '\n')  fputs(prefix, stderr);
+		  if (i > 0 && (*buff)[i-1] == '\n')
+		     /*! prefix? */;
 		  switch (c) {
 		  case '\a' : fputs("\\a", stderr); break;
 		  case '\b' : fputs("\\b", stderr); break;
@@ -976,7 +1155,7 @@ int xiotransfer(xiofile_t *inpipe, xiofile_t *outpipe,
 	       }
 	    } else if (socat_opts.verbhex) {
 	       int i;
-	       fputs(prefix, stderr);
+	       /*! prefix? */
 	       for (i = 0; i < bytes; ++i) {
 		  fprintf(stderr, " %02x", (*buff)[i]);
 	       }
@@ -985,9 +1164,11 @@ int xiotransfer(xiofile_t *inpipe, xiofile_t *outpipe,
 
 	    writt = xiowrite(outpipe, *buff, bytes);
 	    if (writt < 0) {
+#if 0
 	       if (errno == EPIPE) {
 		  return 0;	/* can no longer write; handle like EOF */
 	       }
+#endif
 	       return -1;
 	    } else {
 	       Info3("transferred "F_Zu" bytes from %d to %d",
@@ -1095,7 +1276,7 @@ void socat_signal(int signum) {
 /* this is the callback when the child of an address died */
 static int socat_sigchild(struct single *file) {
    if (file->ignoreeof && !closing) {
-      sleepon = 1;
+      ;
    } else {
       file->eof = MAX(file->eof, 1);
       closing = 1;
@@ -1144,7 +1325,8 @@ static void socat_unlock(void) {
    if (!havelock)  return;
    if (socat_opts.lock.lockfile) {
       if (Unlink(socat_opts.lock.lockfile) < 0) {
-	 Warn1("unlink(\"%s\"): %s", socat_opts.lock.lockfile);
+	 Warn2("unlink(\"%s\"): %s",
+	       socat_opts.lock.lockfile, strerror(errno));
       } else {
 	 Info1("released lock \"%s\"", socat_opts.lock.lockfile);
       }
@@ -1153,7 +1335,7 @@ static void socat_unlock(void) {
 
 /* this is a callback function that may be called by the newchild hook of xio
  */
-static int socat_newchild(pid_t pid) {
+static int socat_newchild(void) {
    havelock = false;
    return 0;
 }
