@@ -1,5 +1,5 @@
-/* $Id: sslcls.c,v 1.5 2004/06/06 17:33:22 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: sslcls.c,v 1.7 2006/06/30 20:46:32 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* explicit system call and C library trace function, for those who miss strace
@@ -165,6 +165,14 @@ int sycSSL_CTX_set_cipher_list(SSL_CTX *ctx, const char *str) {
    return result;
 }
 
+int sycSSL_CTX_set_tmp_dh(SSL_CTX *ctx, DH *dh) {
+   int result;
+   Debug2("SSL_CTX_set_tmp_dh(%p, %p)", ctx, dh);
+   result = SSL_CTX_set_tmp_dh(ctx, dh);
+   Debug1("SSL_CTX_set_tmp_dh() -> %d", result);
+   return result;
+}
+
 int sycSSL_set_cipher_list(SSL *ssl, const char *str) {
    int result;
    Debug2("SSL_set_cipher_list(%p, \"%s\")", ssl, str);
@@ -263,4 +271,40 @@ int sycRAND_egd(const char *path) {
    return result;
 }
 
+DH *sycPEM_read_bio_DHparams(BIO *bp, DH **x, pem_password_cb *cb, void *u) {
+   DH *result;
+   Debug4("PEM_read_bio_DHparams(%p, %p, %p, %p)",
+	  bp, x, cb, u);
+   result = PEM_read_bio_DHparams(bp, x, cb, u);
+   if (result) {
+      /*Debug2("PEM_read_bio_DHparams(, {%p},,) -> %p", *x, result);*/
+      Debug1("PEM_read_bio_DHparams() -> %p", result);
+   } else {
+      Debug("PEM_read_bio_DHparams() -> NULL");
+   }
+   return result;
+}
+
+BIO *sycBIO_new_file(const char *filename, const char *mode) {
+   BIO *result;
+   Debug2("BIO_new_file(\"%s\", \"%s\")", filename, mode);
+   result = BIO_new_file(filename, mode);
+   if (result) {
+      Debug1("BIO_new_file() -> %p", result);
+   } else {
+      Debug("BIO_new_file() -> NULL");
+   }
+   return result;
+}
+
+#if WITH_FIPS
+int sycFIPS_mode_set(int onoff) {
+   int result;
+   Debug1("FIPS_mode_set(%d)", onoff);
+   result = FIPS_mode_set(onoff);
+   Debug1("FIPS_mode_set() -> %d", result);
+   return result;
+}
+#endif /* WITH_FIPS */
+
 #endif /* WITH_SYCLS && WITH_OPENSSL */
