@@ -1,5 +1,5 @@
-/* $Id: sycls.c,v 1.66 2005/08/18 19:44:49 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: sycls.c,v 1.72 2006/07/13 06:35:07 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* explicit system call and C library trace function, for those who miss strace
@@ -580,7 +580,7 @@ int Ioctl(int d, int request, void *argp) {
    if (argp > (void *)0x10000) {	/* fuzzy...*/
       Debug4("ioctl(%d, 0x%x, %p{%lu})", d, request, argp, *(unsigned long *)argp);
    } else {
-      Debug3("ioctl(%d, 0x%x, 0x%x)", d, request, argp);
+      Debug3("ioctl(%d, 0x%x, 0x%p)", d, request, argp);
    }
    retval = ioctl(d, request, argp);
    _errno = errno;
@@ -594,8 +594,6 @@ int Close(int fd) {
    Info1("close(%d)", fd);
    retval = close(fd);
    _errno = errno;
-   if (retval != 0)
-      Warn2("close(%d): %s", fd, strerror(errno));
    Debug1("close()  -> %d", retval);
    errno = _errno;
    return retval;
@@ -688,7 +686,7 @@ int Select(int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
 	   struct timeval *timeout) {
    int result, _errno;
 #if HAVE_FDS_BITS
-   Debug7("select(%d, &0x%lx, &0x%lx, &0x%lx, %s%lu.%06u)",
+   Debug7("select(%d, &0x%lx, &0x%lx, &0x%lx, %s%lu."F_tv_usec")",
 	  n, readfds->fds_bits[0], writefds->fds_bits[0],
 	  exceptfds->fds_bits[0],
 	  timeout?"&":"NULL/", timeout?timeout->tv_sec:0,
@@ -703,7 +701,7 @@ int Select(int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
    result = select(n, readfds, writefds, exceptfds, timeout);
    _errno = errno;
 #if HAVE_FDS_BITS
-   Debug7("select -> (, 0x%lx, 0x%lx, 0x%lx, %s%lu.%06u), %d",
+   Debug7("select -> (, 0x%lx, 0x%lx, 0x%lx, %s%lu."F_tv_usec"), %d",
 	  readfds->fds_bits[0], writefds->fds_bits[0], exceptfds->fds_bits[0],
 	  timeout?"&":"NULL/", timeout?timeout->tv_sec:0,
 	  timeout?timeout->tv_usec:0, result);
@@ -792,7 +790,19 @@ int Link(const char *oldpath, const char *newpath) {
 
 int Execvp(const char *file, char *const argv[]) {
    int result, _errno;
-   Debug2("execvp(%s, %s, ...)", file, argv[0]);
+   if (argv[1] == NULL)
+      Debug2("execvp(\"%s\", \"%s\")", file, argv[0]);
+   else if (argv[2] == NULL)
+      Debug3("execvp(\"%s\", \"%s\" \"%s\")", file, argv[0], argv[1]);
+   else if (argv[3] == NULL)
+      Debug4("execvp(\"%s\", \"%s\" \"%s\" \"%s\")", file, argv[0], argv[1], argv[2]);
+   else if (argv[4] == NULL)
+      Debug5("execvp(\"%s\", \"%s\" \"%s\" \"%s\" \"%s\")", file, argv[0], argv[1], argv[2], argv[3]);
+   else if (argv[5] == NULL)
+      Debug6("execvp(\"%s\", \"%s\" \"%s\" \"%s\" \"%s\" \"%s\")", file, argv[0], argv[1], argv[2], argv[3], argv[4]);
+   else
+      Debug6("execvp(\"%s\", \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" ...)", file, argv[0], argv[1], argv[2], argv[3], argv[4]);
+
    result = execvp(file, argv);
    _errno = errno;
    Debug1("execvp() -> %d", result);
@@ -1015,6 +1025,21 @@ int Recvfrom(int s, void *buf, size_t len, int flags, struct sockaddr *from,
 }
 #endif /* WITH_SOCKET */
 
+#if WITH_SOCKET
+int Recvmsg(int s, struct msghdr *msgh, int flags) {
+   int retval, _errno;
+   char infobuff[256];
+   Debug3("recvmsg(%d, %p, %d)", s, msgh, flags);
+   retval = recvmsg(s, msgh, flags);
+   _errno = errno;
+   Debug2("recvmsg(, {%s}, ) -> %d",
+	  msgh->msg_name?sockaddr_info(msgh->msg_name, infobuff, sizeof(infobuff)):"NULL",
+	  retval);
+   errno = _errno;
+   return retval;
+}
+#endif /* WITH_SOCKET */
+
 #if WITH_SOCKET
 int Send(int s, const void *mesg, size_t len, int flags) {
    int retval, _errno;
@@ -1032,8 +1057,11 @@ int Send(int s, const void *mesg, size_t len, int flags) {
 int Sendto(int s, const void *mesg, size_t len, int flags,
 	   const struct sockaddr *to, socklen_t tolen) {
    int retval, _errno;
-   Debug7("sendto(%d, %p[%08x...], "F_Zu", %d, %p, %d)",
-	  s, mesg, htonl(*(unsigned long *)mesg), len, flags, to, tolen);
+   char infobuff[256];
+
+   sockaddr_info(to, infobuff, sizeof(infobuff));
+   Debug7("sendto(%d, %p[%08x...], "F_Zu", %d, {%s}, %d)",
+	  s, mesg, htonl(*(unsigned long *)mesg), len, flags, infobuff, tolen);
    retval = sendto(s, mesg, len, flags, to, tolen);
    _errno = errno;
    Debug1("sendto() -> %d", retval);
@@ -1062,6 +1090,13 @@ unsigned int Sleep(unsigned int seconds) {
    return retval;
 }
 
+void Usleep(unsigned long usec) {
+   Debug1("usleep(%lu)", usec);
+   usleep(usec);
+   Debug("usleep() ->");
+   return;
+}
+
 #if HAVE_NANOSLEEP
 unsigned int Nanosleep(const struct timespec *req, struct timespec *rem) {
    int retval, _errno;
@@ -1079,6 +1114,16 @@ unsigned int Nanosleep(const struct timespec *req, struct timespec *rem) {
 }
 #endif /* HAVE_NANOSLEEP */
 
+int Pause(void) {
+   int retval, _errno;
+   Debug("pause()");
+   retval = pause();
+   _errno = errno;
+   Debug1("pause() -> %d", retval);
+   errno = _errno;
+   return retval;
+}
+
 #if WITH_IP4 || WITH_IP6
 struct hostent *Gethostbyname(const char *name) {
    struct hostent *hent;
@@ -1109,7 +1154,11 @@ int Getaddrinfo(const char *node, const char *service,
 	   hints->ai_canonname, hints->ai_next, res);
    result = getaddrinfo(node, service, hints, res);
    if (result == 0) {
-      Debug("getaddrinfo(,,,{}) -> 0");
+      char sockbuff[256];
+      sockaddr_info((*res)->ai_addr, sockbuff, sizeof(sockbuff));
+      Debug2("getaddrinfo(,,,{{%s, %s}) -> 0",
+	     sockbuff,
+	    (*res)->ai_canonname?(*res)->ai_canonname:"");
    } else {
       Debug2("getaddrinfo(,,,{%p}) -> %d", *res, result);
    }
@@ -1329,6 +1378,11 @@ void Exit(int status) {
    exit(status);
 }
 
+void Abort(void) {
+   Debug("abort()");
+   abort();
+}
+
 int Mkstemp(char *template) {
    int result, _errno;
    Debug1("mkstemp(\"%s\")", template);
