@@ -1,5 +1,5 @@
-/* $Id: sycls.h,v 1.43 2005/03/15 07:32:52 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: sycls.h,v 1.48 2006/07/13 06:35:55 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __sycls_h_included
@@ -85,6 +85,7 @@ int Sigaction(int signum, const struct sigaction *act,
 	      struct sigaction *oldact);
 unsigned int Alarm(unsigned int seconds);
 int Kill(pid_t pid, int sig);
+int Link(const char *oldpath, const char *newpath);
 int Execvp(const char *file, char *const argv[]);
 int System(const char *string);
 int Socketpair(int d, int type, int protocol, int sv[2]);
@@ -101,13 +102,16 @@ int Setsockopt(int s, int level, int optname, const void *optval, int optlen);
 int Recv(int s, void *buf, size_t len, int flags);
 int Recvfrom(int s, void *buf, size_t len, int flags, struct sockaddr *from,
 	     socklen_t *fromlen);
+int Recvmsg(int s, struct msghdr *msg, int flags);
 int Send(int s, const void *mesg, size_t len, int flags);
 int Sendto(int s, const void *msg, size_t len, int flags,
 	   const struct sockaddr *to, socklen_t tolen);
 int Shutdown(int fd, int how);
 #endif /* WITH_SOCKET */
 unsigned int Sleep(unsigned int seconds);
+void Usleep(unsigned long usec);
 unsigned int Nanosleep(const struct timespec *req, struct timespec *rem);
+int Pause(void);
 struct hostent *Gethostbyname(const char *name);
 int Getaddrinfo(const char *node, const char *service,
 		const struct addrinfo *hints, struct addrinfo **res);
@@ -130,6 +134,8 @@ int Gethostname(char *name, size_t len);
 int Uname(struct utsname *buf);
 int Atexit(void (*func)(void));
 void Exit(int status);
+void Abort(void);
+int Mkstemp(char *template);
 
 char *Readline(const char *prompt);
 void Using_history(void);
@@ -204,6 +210,7 @@ void Add_history(const char *string);
 #define Sigaction(s,a,o) sigaction(s,a,o)
 #define Alarm(s) alarm(s)
 #define Kill(p,s) kill(p,s)
+#define Link(o,n) link(o,n)
 #define Execvp(f,a) execvp(f,a)
 #define System(s) system(s)
 #define Socketpair(d,t,p,s) socketpair(d,t,p,s)
@@ -216,13 +223,16 @@ void Add_history(const char *string);
 #define Getpeername(s,n,l) getpeername(s,n,l)
 #define Getsockopt(s,d,n,v,l) getsockopt(s,d,n,v,l)
 #define Setsockopt(s,d,n,v,l) setsockopt(s,d,n,v,l)
-#define Recv(b,l,f) recv(b,l,f)
+#define Recv(s,b,l,f) recv(s,b,l,f)
 #define Recvfrom(s,b,bl,f,fr,fl) recvfrom(s,b,bl,f,fr,fl)
+#define Recvmsg(s,m,f) recvmsg(s,m,f)
 #define Send(s,m,l,f) send(s,m,l,f)
 #define Sendto(s,b,bl,f,t,tl) sendto(s,b,bl,f,t,tl)
 #define Shutdown(f,h) shutdown(f,h)
 #define Sleep(s) sleep(s)
+#define Usleep(u) usleep(u)
 #define Nanosleep(req,rem) nanosleep(req,rem)
+#define Pause() pause()
 #define Gethostbyname(n) gethostbyname(n)
 #define Getaddrinfo(n,s,h,r) getaddrinfo(n,s,h,r)
 #define Getipnodebyname(n,a,f,e) getipnodebyname(n,a,f,e)
@@ -242,6 +252,8 @@ void Add_history(const char *string);
 #define Uname(b) uname(b)
 #define Atexit(f) atexit(f)
 #define Exit(s) exit(s)
+#define Abort() abort()
+#define Mkstemp(t) mkstemp(t)
 
 #define Readline(p) readline(p)
 #define Using_history() using_history()
