@@ -1,5 +1,5 @@
-/* $Id: sysincludes.h,v 1.15 2004/10/24 13:32:42 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: sysincludes.h,v 1.19 2006/05/31 19:13:33 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __sysincludes_h_included
@@ -55,6 +55,9 @@
 #if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>	/* struct sockaddr, struct linger, socket(), connect() */
 #endif
+#if HAVE_SYS_UIO_H
+#include <sys/uio.h>	/* struct iovec */
+#endif
 #if HAVE_SYS_STAT_H
 #include <sys/stat.h>	/* struct stat, stat(), open() */
 #endif
@@ -101,6 +104,9 @@
 #include <netinet6/in6.h>
 #  endif
 #include <arpa/inet.h>	/* Linux: inet_aton() */
+#if HAVE_RESOLV_H
+#include <resolv.h>	/* _res */
+#endif
 #endif /* WITH_IP4 || WITH_IP6 */
 #if HAVE_TERMIOS_H && WITH_TERMIOS
 #include <termios.h>
@@ -120,6 +126,9 @@
 #if HAVE_REGEX_H
 #include <regex.h>
 #endif
+#if HAVE_LINUX_EXT2_FS_H
+#include <linux/ext2_fs.h>	/* Linux ext2 filesystem definitions */
+#endif
 #if WITH_READLINE
 #  if HAVE_READLINE_READLINE_H
 #include <readline/readline.h>
