@@ -1,5 +1,5 @@
-/* $Id: xio-fdnum.h,v 1.4 2001/11/04 17:19:20 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001 */
+/* $Id: xio-fdnum.h,v 1.6 2006/03/21 20:48:31 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_fdnum_h_included
@@ -7,6 +7,6 @@
 
 extern const struct addrdesc addr_fd;
 
-extern int xioopen_fd(char *arg, int rw, xiofile_t *fd, unsigned groups, int numfd, int dummy2, void *dummyp1);
+extern int xioopen_fd(struct opt *opts, int rw, xiosingle_t *xfd, int numfd, int dummy2, int dummy3);
 
 #endif /* !defined(__xio_fdnum_h_included) */
