@@ -1,5 +1,5 @@
-/* $Id: xio-file.c,v 1.15 2005/03/13 12:21:29 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: xio-file.c,v 1.20 2006/03/21 20:38:08 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses of open type */
@@ -11,7 +11,7 @@
 #include "xio-file.h"
 
 
-static int xioopen_open(char *a1, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, void *dummyp1);
+static int xioopen_open(int argc, const char *argv[], struct opt *opts, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, int dummy3);
 
 
 #if WITH_OPEN
@@ -66,32 +66,35 @@ const struct optdesc opt_o_trunc     = { "o-trunc",     "trunc",  OPT_O_TRUNC,
 
 #if WITH_FILE	/*! inconsistent name FILE vs. OPEN */
 
-const struct addrdesc addr_open   = { "open",   3, xioopen_open, GROUP_FD|GROUP_NAMED|GROUP_OPEN|GROUP_FILE, 0, 0, NULL HELP(":<filename>") };
-
+const struct addrdesc addr_open   = { "open",   3, xioopen_open, GROUP_FD|GROUP_FIFO|GROUP_CHR|GROUP_BLK|GROUP_REG|GROUP_NAMED|GROUP_OPEN|GROUP_FILE|GROUP_TERMIOS, 0, 0, 0 HELP(":<filename>") };
 
 /* open for writing:
    if the filesystem entry already exists, the data is appended
    if it does not exist, a file is created and the data is appended
 */
-static int xioopen_open(char *a1, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, void *dummyp1) {
+static int xioopen_open(int argc, const char *argv[], struct opt *opts, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, int dummy3) {
+   const char *filename = argv[1];
    int rw = (xioflags & XIO_ACCMODE);
-   char *a2;
-   struct opt *opts = NULL;
    bool exists;
+   bool opt_unlink_close = false;
    int result;
 
-   if ((a2 = strchr(a1, ',')) != NULL) {
-      *a2++ = '\0';
-   }
-
    /* remove old file, or set user/permissions on old file; parse options */
-   if ((result = _xioopen_named_early(a1, a2, groups, &exists, &opts)) < 0) {
+   if ((result = _xioopen_named_early(argc, argv, fd, groups, &exists, opts)) < 0) {
       return result;
    }
 
+   retropt_bool(opts, OPT_UNLINK_CLOSE, &opt_unlink_close);
+   if (opt_unlink_close) {
+      if ((fd->stream.unlink_close = strdup(filename)) == NULL) {
+	 Error1("strdup(\"%s\"): out of memory", filename);
+      }
+      fd->stream.opt_unlink_close = true;
+   }
+
    Notice3("opening %s \"%s\" for %s",
-	   filetypenames[(result&S_IFMT)>>12], a1, ddirection[rw]);
-   if ((result = _xioopen_open(a1, rw, opts)) < 0)
+	   filetypenames[(result&S_IFMT)>>12], filename, ddirection[rw]);
+   if ((result = _xioopen_open(filename, rw, opts)) < 0)
       return result;
    fd->stream.fd = result;
 
@@ -106,7 +109,7 @@ static int xioopen_open(char *a1, int xioflags, xiofile_t *fd, unsigned groups,
    }
 #endif /* WITH_TERMIOS */
 
-   applyopts_named(a1, opts, PH_FD);
+   applyopts_named(filename, opts, PH_FD);
    applyopts(fd->stream.fd, opts, PH_FD);
    applyopts_cloexec(fd->stream.fd, opts);
 
