@@ -1,5 +1,5 @@
-/* $Id: xio-gopen.c,v 1.19 2005/03/13 12:21:41 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: xio-gopen.c,v 1.27 2006/05/31 19:27:55 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses of generic open type */
@@ -13,26 +13,21 @@
 
 #if WITH_GOPEN
 
-static int xioopen_gopen(char *a1, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, void *dummyp1);
+static int xioopen_gopen(int argc, const char *argv[], struct opt *opts, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, int dummy3);
 
 
-const struct addrdesc addr_gopen  = { "gopen",  3, xioopen_gopen, GROUP_FD|GROUP_NAMED|GROUP_OPEN|GROUP_FILE|GROUP_SOCKET|GROUP_SOCK_UNIX, 0, 0, NULL HELP(":<filename>") };
+const struct addrdesc addr_gopen  = { "gopen",  3, xioopen_gopen, GROUP_FD|GROUP_FIFO|GROUP_CHR|GROUP_BLK|GROUP_REG|GROUP_NAMED|GROUP_OPEN|GROUP_FILE|GROUP_TERMIOS|GROUP_SOCKET|GROUP_SOCK_UNIX, 0, 0, 0 HELP(":<filename>") };
 
-
-static int xioopen_gopen(char *a1, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, void *dummyp1) {
-   char *a2;
-   struct opt *opts = NULL;
+static int xioopen_gopen(int argc, const char *argv[], struct opt *opts, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, int dummy3) {
+   const char *filename = argv[1];
    flags_t openflags = (xioflags & XIO_ACCMODE);
    mode_t st_mode;
    bool exists;
+   bool opt_unlink_close = false;
    int result;
 
-   if ((a2 = strchr(a1, ',')) != NULL) {
-      *a2++ = '\0';
-   }
-
    if ((result =
-	  _xioopen_named_early(a1, a2, GROUP_NAMED|groups, &exists, &opts)) < 0) {
+	  _xioopen_named_early(argc, argv, fd, GROUP_NAMED|groups, &exists, opts)) < 0) {
       return result;
    }
    st_mode = result;
@@ -53,22 +48,30 @@ static int xioopen_gopen(char *a1, int xioflags, xiofile_t *fd, unsigned groups,
       int optsotype = -1;
       struct sockaddr_un sa, us;
       bool needbind = false;
-      socklen_t uslen;
+      socklen_t uslen = sizeof(us);
       char infobuff[256];
       struct opt *opts2;
 
       socket_un_init(&sa);
       socket_un_init(&us);
 
-      Info1("\"%s\" is a socket, connecting to it", a1);
+      Info1("\"%s\" is a socket, connecting to it", filename);
       if (retropt_int(opts, OPT_SO_TYPE, &optsotype) == 0) {
 	 socktype = optsotype;
       }
 
-      if (retropt_bind(opts, AF_UNIX, NULL, (struct sockaddr *)&us, 0) >= 0) {
+      if (retropt_bind(opts, AF_UNIX, socktype, 0, (struct sockaddr *)&us, &uslen, 0) >= 0) {
 	 needbind = true;
       }
 
+      retropt_bool(opts, OPT_UNLINK_CLOSE, &opt_unlink_close);
+      if (opt_unlink_close) {
+	 if ((fd->stream.unlink_close = strdup(filename)) == NULL) {
+	    Error1("strdup(\"%s\"): out of memory", filename);
+	 }
+	 fd->stream.opt_unlink_close = true;
+      }
+
       /* save options, because we might have to start again with Socket() */
       opts2 = copyopts(opts, GROUP_ALL);
 
@@ -83,7 +86,7 @@ static int xioopen_gopen(char *a1, int xioflags, xiofile_t *fd, unsigned groups,
       applyopts_cloexec(fd->stream.fd, opts);
 
       sa.sun_family = AF_UNIX;
-      strncpy(sa.sun_path, a1, UNIX_PATH_MAX);
+      strncpy(sa.sun_path, filename, sizeof(sa.sun_path));
 
 #if 0
       applyopts(fd->stream.fd, opts, PH_PREBIND);
@@ -129,9 +132,9 @@ static int xioopen_gopen(char *a1, int xioflags, xiofile_t *fd, unsigned groups,
 	    applyopts_cloexec(fd->stream.fd, opts);
 
 	    sa.sun_family = AF_UNIX;
-	    strncpy(sa.sun_path, a1, UNIX_PATH_MAX);
+	    strncpy(sa.sun_path, filename, sizeof(sa.sun_path));
 	    
-	    fd->stream.dtype = DATA_RECV;
+	    fd->stream.dtype = DATA_RECVFROM;
 	    fd->stream.salen = sizeof(sa);
 	    memcpy(&fd->stream.peersa.soa, &sa, fd->stream.salen);
 	 } else {
@@ -146,7 +149,7 @@ static int xioopen_gopen(char *a1, int xioflags, xiofile_t *fd, unsigned groups,
       applyopts_fchown(fd->stream.fd, opts);
       applyopts(fd->stream.fd, opts, PH_CONNECTED);
       applyopts(fd->stream.fd, opts, PH_LATE);
-      applyopts_named(a1, opts, PH_PASTOPEN);	/* unlink-late */
+      applyopts_named(filename, opts, PH_PASTOPEN);	/* unlink-late */
 
       if (Getsockname(fd->stream.fd, (struct sockaddr *)&us, &uslen) < 0) {
 	 Warn4("getsockname(%d, %p, {%d}): %s",
@@ -163,13 +166,19 @@ static int xioopen_gopen(char *a1, int xioflags, xiofile_t *fd, unsigned groups,
    } else {
       /* a file name */
 
-      /*applyopts_early(a1, opts);*/
-      applyopts_named(a1, opts, PH_EARLY);
+      Info1("\"%s\" is not a socket, open()'ing it", filename);
+
+      retropt_bool(opts, OPT_UNLINK_CLOSE, &opt_unlink_close);
+      if (opt_unlink_close) {
+	 if ((fd->stream.unlink_close = strdup(filename)) == NULL) {
+	    Error1("strdup(\"%s\"): out of memory", filename);
+	 }
+	 fd->stream.opt_unlink_close = true;
+      }
 
-      Debug1("\"%s\" is not a socket, opening it", a1);
       Notice3("opening %s \"%s\" for %s",
-	      filetypenames[(st_mode&S_IFMT)>>12], a1, ddirection[(xioflags&XIO_ACCMODE)]);
-      if ((result = _xioopen_open(a1, openflags, opts)) < 0)
+	      filetypenames[(st_mode&S_IFMT)>>12], filename, ddirection[(xioflags&XIO_ACCMODE)]);
+      if ((result = _xioopen_open(filename, openflags, opts)) < 0)
 	 return result;
 #if __unix && __sun /* solaris */
       if (S_ISCHR(st_mode)) {
@@ -190,7 +199,7 @@ static int xioopen_gopen(char *a1, int xioflags, xiofile_t *fd, unsigned groups,
 	 }
       }
 #endif /* WITH_TERMIOS */
-      applyopts_named(a1, opts, PH_FD);
+      applyopts_named(filename, opts, PH_FD);
       applyopts(fd->stream.fd, opts, PH_FD);
       applyopts_cloexec(fd->stream.fd, opts);
    }
