@@ -1,5 +1,5 @@
-/* $Id: xio-ip.h,v 1.5 2003/02/23 08:33:41 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001, 2003 */
+/* $Id: xio-ip.h,v 1.8 2006/05/19 05:54:34 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_ip_h_included
@@ -23,7 +23,20 @@ extern const struct optdesc opt_ip_multicast_ttl;
 extern const struct optdesc opt_ip_multicast_loop;
 extern const struct optdesc opt_ip_pktoptions;
 
-extern int parseipaddr(char *addr, struct sockaddr *out, const char *protoname,
-		int withaddr, int withport);
+extern const struct optdesc opt_res_debug;
+extern const struct optdesc opt_res_aaonly;
+extern const struct optdesc opt_res_usevc;
+extern const struct optdesc opt_res_primary;
+extern const struct optdesc opt_res_igntc;
+extern const struct optdesc opt_res_recurse;
+extern const struct optdesc opt_res_defnames;
+extern const struct optdesc opt_res_stayopen;
+extern const struct optdesc opt_res_dnsrch;
+
+extern int xiogetaddrinfo(const char *node, const char *service,
+			  int family, int socktype, int protocol,
+			  union sockaddr_union *sa, socklen_t *socklen,
+			  unsigned long res_opts0, unsigned long res_opts1);
+extern int parserange(char *rangename, int pf, union xiorange_union *range);
 
 #endif /* !defined(__xio_ip_h_included) */
