@@ -1,40 +1,30 @@
-/* $Id: xio-listen.c,v 1.35 2006/01/29 16:33:07 gerhard Exp $ */
+/* $Id: xio-listen.c,v 1.41 2006/07/13 06:43:32 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for listen socket options */
 
 #include "xiosysincludes.h"
-#if WITH_LIBWRAP
-#include "tcpd.h"
-#endif
-#include "xioopen.h"
-
-#include "xio-named.h"
-#include "xio-ip.h"
-#include "xio-ip4.h"
-#include "xio-listen.h"
 
 #if WITH_LISTEN
 
-/***** LISTEN addresses *****/
+#include "xioopen.h"
+#include "xio-named.h"
+#include "xio-socket.h"
+#include "xio-ip.h"
+#include "xio-ip4.h"
+#include "xio-listen.h"
+#include "xio-tcpwrap.h"
+
+/***** LISTEN options *****/
 const struct optdesc opt_backlog = { "backlog",   NULL, OPT_BACKLOG,     GROUP_LISTEN, PH_LISTEN, TYPE_INT,    OFUNC_SPEC };
 const struct optdesc opt_fork    = { "fork",      NULL, OPT_FORK,        GROUP_CHILD,   PH_PASTACCEPT, TYPE_BOOL,  OFUNC_SPEC };
 /**/
 #if (WITH_UDP || WITH_TCP)
 const struct optdesc opt_range   = { "range",     NULL, OPT_RANGE,       GROUP_RANGE,  PH_ACCEPT, TYPE_STRING, OFUNC_SPEC };
 #endif
-#if (WITH_TCP || WITH_UDP) && WITH_LIBWRAP
-const struct optdesc opt_tcpwrappers = { "tcpwrappers", "tcpwrap", OPT_TCPWRAPPERS, GROUP_RANGE,  PH_ACCEPT, TYPE_STRING_NULL, OFUNC_SPEC };
-#endif
 
 
-#if (WITH_TCP || WITH_UDP) && WITH_LIBWRAP
-/* libwrap these are declared only extern and would be unresolved without these
-   definitions */
-int allow_severity=10, deny_severity=10;
-#endif
-
 int
    xioopen_listen(struct single *xfd, int xioflags,
 		  struct sockaddr *us, socklen_t uslen,
@@ -108,18 +98,6 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
    int backlog = 5;	/* why? 1 seems to cause problems under some load */
    bool dofork = false;
    pid_t pid;	/* mostly int; only used with fork */
-   char *rangename;	/* allowed client address range - text form */
-   unsigned short sourceport;
-   bool lowport = false;
-   bool dosourceport = false;
-#if (WITH_TCP || WITH_UDP) && WITH_LIBWRAP
-   bool dolibwrap = false;
-   char *libwrapname = NULL;
-#endif
-#if WITH_IP4
-   byte4_t netaddr_in = 0;
-   byte4_t netmask_in = 0;
-#endif /* WITH_IP4 */
    char infobuff[256];
    char lisname[256];
    int result;
@@ -188,8 +166,8 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
       fields that we want to know. */
    salen = sizeof(sa);
    if (Getsockname(xfd->fd, us, &uslen) < 0) {
-      Error4("getsockname(%d, %p, {%d}): %s",
-	     xfd->fd, &us, uslen, strerror(errno));
+      Warn4("getsockname(%d, %p, {%d}): %s",
+	    xfd->fd, &us, uslen, strerror(errno));
    }
 
    applyopts(xfd->fd, opts, PH_PASTBIND);
@@ -208,28 +186,23 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
    }
 
 #if WITH_IP4 /*|| WITH_IP6*/
-   if (retropt_string(opts, OPT_RANGE, &rangename) >= 0) {
-      if (pf == PF_INET) {
-	 parserange(rangename, us->sa_family, &netaddr_in, &netmask_in);
-      } else {
-	 Error1("range option not supported with address family %d", pf);
+   if (retropt_string(opts, OPT_RANGE, &xfd->para.socket.rangename) >= 0) {
+      if (parserange(xfd->para.socket.rangename, us->sa_family,
+		     &xfd->para.socket.range) < 0) {
 	 return STAT_NORETRY;
       }
    }
 #endif
+
 #if (WITH_TCP || WITH_UDP) && WITH_LIBWRAP
-   if (retropt_string(opts, OPT_TCPWRAPPERS, &libwrapname) >= 0) {
-      dolibwrap = true;
-      if (libwrapname == NULL) {
-	 libwrapname = (char *)diag_get_string('p');
-      }
-   }
+   xio_retropt_tcpwrap(xfd, opts);
 #endif /* && (WITH_TCP || WITH_UDP) && WITH_LIBWRAP */
+
 #if WITH_TCP || WITH_UDP
-   if (retropt_ushort(opts, OPT_SOURCEPORT, &sourceport) >= 0) {
-      dosourceport = true;
+   if (retropt_ushort(opts, OPT_SOURCEPORT, &xfd->para.socket.ip.sourceport) >= 0) {
+      xfd->para.socket.ip.dosourceport = true;
    }
-   retropt_bool(opts, OPT_LOWPORT, &lowport);
+   retropt_bool(opts, OPT_LOWPORT, &xfd->para.socket.ip.lowport);
 #endif /* WITH_TCP || WITH_UDP */
 
    if (xioopts.logopt == 'm') {
@@ -244,7 +217,7 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
       int ps;		/* peer socket */
       char _peername[SOCKADDR_MAX];
       union sockaddr_union _sockname;
-      struct sockaddr *pa = (struct sockaddr *)_peername;	/* peer address */
+      union sockaddr_union *pa = (union sockaddr_union *)_peername;	/* peer address */
       union sockaddr_union *la = &_sockname;	/* local address */
       socklen_t pas = sizeof(_peername);	/* peer address size */
       socklen_t las = sizeof(_sockname);	/* local address size */
@@ -272,124 +245,28 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
 	 return STAT_RETRYLATER;
       } while (true);
       applyopts_cloexec(ps, opts);
-      if (Getpeername(ps, pa, &pas) < 0) {
+      if (Getpeername(ps, &pa->soa, &pas) < 0) {
 	 Warn4("getpeername(%d, %p, {"F_socklen"}): %s",
 	       ps, pa, pas, strerror(errno));
       }
       if (Getsockname(ps, &la->soa, &las) < 0) {
-	 Warn4("getsockname(%d, %d, {"F_socklen"}): %s",
+	 Warn4("getsockname(%d, %p, {"F_socklen"}): %s",
 	       ps, pa, pas, strerror(errno));
       }
       Notice2("accepting connection from %s on %s",
-	      sockaddr_info(pa, peername, sizeof(peername)),
+	      sockaddr_info(&pa->soa, peername, sizeof(peername)),
 	      sockaddr_info(&la->soa, sockname, sizeof(sockname)));
 
-#if WITH_IP4
-      if (rangename != NULL) {
-	 if (checkrange(pa, &netaddr_in, &netmask_in) < 0) {
-	    char infobuff[256];
-	    Warn1("refusing connection from %s due to range option",
-		  sockaddr_info((struct sockaddr *)pa,
-				infobuff, sizeof(infobuff)));
-	    if (Shutdown(ps, 2) < 0) {
-	       Warn2("shutdown(%d, 2): %s", pa, strerror(errno));
-	    }
-	    continue;
+      if (xiocheckpeer(xfd, pa, la) < 0) {
+	 if (Shutdown(ps, 2) < 0) {
+	    Info2("shutdown(%d, 2): %s", ps, strerror(errno));
 	 }
-	 Info1("permitting connection from %s due to range option",
-	       sockaddr_info((struct sockaddr *)pa,
-			     infobuff, sizeof(infobuff)));
+	 continue;
       }
-#endif /* WITH_IP4 */
 
-#if (WITH_TCP || WITH_UDP) && WITH_LIBWRAP
-      if (dolibwrap || libwrapname) {
-	 struct request_info ri;
-	 int allow;
-
-	 Debug5("request_init(%p, RQ_FILE, %d, RQ_CLIENT_SIN, %p, RQ_SERVER_SIN, %p, RQ_DAEMON, \"%s\", 0",
-		&ri, xfd->fd, &la->soa, pa, libwrapname);
-	 request_init(&ri, RQ_FILE, xfd->fd, RQ_CLIENT_SIN, &la->soa,
-		      RQ_SERVER_SIN, pa, RQ_DAEMON, libwrapname, 0);
-	 Debug("request_init() ->");
-
-	 Debug1("hosts_access(%p)", &ri);
-	 allow = hosts_access(&ri);
-	 Debug1("hosts_access() -> %d", allow);
-	 if (allow == 0) {
-	    char infobuff[256];
-	    Warn1("refusing connection from %s due to tcpwrapper option",
-		  sockaddr_info((struct sockaddr *)pa,
-				infobuff, sizeof(infobuff)));
-	    if (Shutdown(ps, 2) < 0) {
-	       Warn2("shutdown(%d, 2): %s", pa, strerror(errno));
-	    }
-	    continue;
-	 }
-	 Info1("permitting connection from %s due to tcpwrapper option",
-	       sockaddr_info((struct sockaddr *)pa,
-			     infobuff, sizeof(infobuff)));
-      }
-#endif /* (WITH_TCP || WITH_UDP) && WITH_LIBWRAP */
-
-#if WITH_TCP || WITH_UDP
-      if (dosourceport) {
-#if WITH_IP4
-	 if (pa->sa_family == AF_INET &&
-	     ntohs(((struct sockaddr_in *)pa)->sin_port) != sourceport) {
-	    Warn1("refusing connection from %s due to sourceport option",
-		  sockaddr_info((struct sockaddr *)pa,
-				infobuff, sizeof(infobuff)));
-	    if (Shutdown(ps, 2) < 0) {
-	       Warn2("shutdown(%d, 2): %s", pa, strerror(errno));
-	    }
-	    continue;
-	 }
-#endif /* WITH_IP4 */
-#if WITH_IP6
-	 if (pa->sa_family == AF_INET6 &&
-	     ntohs(((struct sockaddr_in6 *)pa)->sin6_port) != sourceport) {
-	    Warn1("refusing connection from %s due to sourceport option",
-		   sockaddr_info((struct sockaddr *)pa,
-				 infobuff, sizeof(infobuff)));
-	    if (Shutdown(ps, 2) < 0) {
-	       Warn2("shutdown(%d, 2): %s", pa, strerror(errno));
-	    }
-	    continue;
-	 }
-#endif /* WITH_IP6 */
-	 Info1("permitting connection from %s due to sourceport option",
-	       sockaddr_info((struct sockaddr *)pa,
-			     infobuff, sizeof(infobuff)));
-      } else if (lowport) {
-	 if (pa->sa_family == AF_INET &&
-	     ntohs(((struct sockaddr_in *)pa)->sin_port) >= IPPORT_RESERVED) {
-	    Warn1("refusing connection from %s due to lowport option",
-		  sockaddr_info((struct sockaddr *)pa,
-				infobuff, sizeof(infobuff)));
-	    if (Shutdown(ps, 2) < 0) {
-	       Warn2("shutdown(%d, 2): %s", pa, strerror(errno));
-	    }
-	    continue;
-	 }
-#if WITH_IP6
-	 else if (pa->sa_family == AF_INET6 &&
-		  ntohs(((struct sockaddr_in6 *)pa)->sin6_port) >=
-		  IPPORT_RESERVED) {
-	    Warn1("refusing connection from %s due to lowport option",
-		  sockaddr_info((struct sockaddr *)pa,
-				infobuff, sizeof(infobuff)));
-	    if (Shutdown(ps, 2) < 0) {
-	       Warn2("shutdown(%d, 2): %s", pa, strerror(errno));
-	    }
-	    continue;
-	 }
-#endif /* WITH_IP6 */
-	 Info1("permitting connection from %s due to lowport option",
-	       sockaddr_info((struct sockaddr *)pa,
-			     infobuff, sizeof(infobuff)));
-      }
-#endif /* WITH_TCP || WITH_UDP */
+      Info1("permitting connection from %s",
+	    sockaddr_info((struct sockaddr *)pa,
+			  infobuff, sizeof(infobuff)));
 
       applyopts(xfd->fd, opts, PH_FD);
 
@@ -403,7 +280,7 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
 	 }
 	 if (pid == 0) {	/* child */
 	    if (Close(xfd->fd) < 0) {
-	       Warn2("close(%d): %s", xfd->fd, strerror(errno));
+	       Info2("close(%d): %s", xfd->fd, strerror(errno));
 	    }
 	    xfd->fd = ps;
 
@@ -414,12 +291,9 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
 	    level = E_ERROR;
 #endif /* WITH_RETRY */
 
-	    xiodroplocks();	/* drop locks in xio addresses */
-	    /* some locks belong to parent, so "drop" them now */
-	    if (xiohook_newchild) {
-	       if ((*xiohook_newchild)(pid) != 0) {
-		  Exit(1);
-	       }
+	    /* drop parents locks, reset FIPS... */
+	    if (xio_forked_inchild() != 0) {
+	       Exit(1);
 	    }
 
 #if WITH_UNIX
@@ -435,13 +309,13 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
 	 /* shutdown() closes the socket even for the child process, but
 	    close() does what we want */
 	 if (Close(ps) < 0) {
-	    Warn2("close(%d): %s", ps, strerror(errno));
+	    Info2("close(%d): %s", ps, strerror(errno));
 	 }
 	 Notice1("forked off child process "F_pid, pid);
 	 Info("still listening");
       } else {
 	 if (Close(xfd->fd) < 0) {
-	    Warn2("close(%d): %s", xfd->fd, strerror(errno));
+	    Info2("close(%d): %s", xfd->fd, strerror(errno));
 	 }
 	 xfd->fd = ps;
 	break;
