@@ -1,16 +1,16 @@
-/* $Id: xio-named.c,v 1.21 2005/09/03 16:51:36 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: xio-named.c,v 1.27 2006/07/13 06:45:49 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for filesystem entry functions */
 
 #include "xiosysincludes.h"
-#include "xioopen.h"
-
-#include "xio-named.h"
 
 #if WITH_NAMED
 
+#include "xioopen.h"
+#include "xio-named.h"
+
 
 const struct optdesc opt_group_early = { "group-early", NULL, OPT_GROUP_EARLY, GROUP_NAMED, PH_PREOPEN, TYPE_GIDT, OFUNC_SPEC };
 const struct optdesc opt_perm_early  = { "perm-early",  NULL, OPT_PERM_EARLY,  GROUP_NAMED, PH_PREOPEN, TYPE_MODET,OFUNC_SPEC };
@@ -90,8 +90,10 @@ int applyopts_named(const char *filename, struct opt *opts, unsigned int phase)
    If the path exists, its st_mode field is returned.
    After this sub you may proceed with open() or whatever...
    */
-int _xioopen_named_early(const char *path, char *addropts, int groups,
-		      bool *exists, struct opt **opts) {
+int _xioopen_named_early(int argc, const char *argv[], xiofile_t *xfd,
+			 int groups,
+		      bool *exists, struct opt *opts) {
+   const char *path = argv[1];
    unsigned int iogroups = 0;
 #if HAVE_STAT64
    struct stat64 statbuf;
@@ -100,6 +102,9 @@ int _xioopen_named_early(const char *path, char *addropts, int groups,
 #endif /* !HAVE_STAT64 */
    bool opt_unlink_early = false;
 
+   if (argc != 2) {
+      Error2("%s: wrong number of parameters (%d instead of 1)", argv[0]?argv[0]:"<named>", argc);
+   }
    statbuf.st_mode = 0;
    /* find the appropriate groupbits */
    if (
@@ -113,22 +118,19 @@ int _xioopen_named_early(const char *path, char *addropts, int groups,
 	 Error2("stat(\"%s\"): %s", path, strerror(errno));
 	    return STAT_RETRYLATER;
       }
-      iogroups = 0;
+      iogroups = GROUP_REG;
       *exists = false;
    } else {
       iogroups = _groupbits(statbuf.st_mode);
       *exists = true;
    }
 
-   if (parseopts(addropts, GROUP_FD|iogroups|groups, opts)
-       < 0) {
-      return STAT_NORETRY;
-   }
-   applyopts(-1, *opts, PH_INIT);
+   if (applyopts_single(&xfd->stream, opts, PH_INIT) < 0)  return -1;
+   applyopts(-1, opts, PH_INIT);
 
-   retropt_bool(*opts, OPT_UNLINK_EARLY, &opt_unlink_early);
+   retropt_bool(opts, OPT_UNLINK_EARLY, &opt_unlink_early);
    if (*exists && opt_unlink_early) {
-      Debug1("\"%s\" already exists; removing it", path);
+      Info1("\"%s\" already exists; removing it", path);
       if (Unlink(path) < 0) {
 	 Error2("unlink(\"%s\"): %s", path, strerror(errno));
 	 *exists = true;
@@ -137,12 +139,12 @@ int _xioopen_named_early(const char *path, char *addropts, int groups,
       }
    }
 
-   /*applyopts_early(path, *opts);*/
-   applyopts_named(path, *opts, PH_EARLY);
+   applyopts(-1, opts, PH_EARLY);
+   applyopts_named(path, opts, PH_EARLY);
    if (*exists) {
-      applyopts_named(path, *opts, PH_PREOPEN);
+      applyopts_named(path, opts, PH_PREOPEN);
    } else {
-      dropopts(*opts, PH_PREOPEN);
+      dropopts(opts, PH_PREOPEN);
    }
 
    return statbuf.st_mode;
@@ -187,6 +189,10 @@ int _xioopen_open(const char *path, int rw, struct opt *opts) {
    if (retropt_bool(opts, OPT_O_NOINHERIT, &flag) >= 0   && flag)
       flags |= O_NOINHERIT;
 #endif
+#ifdef O_NOATIME
+   if (retropt_bool(opts, OPT_O_NOATIME,   &flag) >= 0   && flag)
+      flags |= O_NOATIME;
+#endif
 
    retropt_modet(opts, OPT_PERM,      &mode);
 
