@@ -1,5 +1,5 @@
-/* $Id: xio-named.h,v 1.6 2005/09/03 16:51:36 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: xio-named.h,v 1.8 2006/03/18 20:04:31 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_named_h_included
@@ -18,8 +18,9 @@ extern const struct optdesc opt_umask;
 
 extern int
    applyopts_named(const char *filename, struct opt *opts, unsigned int phase);
-extern int _xioopen_named_early(const char *path, char *addropts, int groups,
-		      bool *exists, struct opt **opts);
+extern int _xioopen_named_early(int argc, const char *argv[], xiofile_t *xfd,
+				int groups,
+				bool *exists, struct opt *opts);
 extern int _xioopen_open(const char *path, int rw, struct opt *opts);
 
 #endif /* !defined(__xio_named_h_included) */
