@@ -1,4 +1,4 @@
-/* $Id: xio-openssl.c,v 1.20 2006/01/29 16:34:02 gerhard Exp $ */
+/* $Id: xio-openssl.c,v 1.28 2006/06/30 20:51:56 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2002-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -36,12 +36,14 @@
  */
 
 /* static declaration of ssl's open function */
-static int xioopen_openssl_connect(char *arg, int xioflags, xiofile_t *fd, unsigned groups,
-			   int dummy1, int dummy2, void *dummyp1);
+static int xioopen_openssl_connect(int argc, const char *argv[], struct opt *opts,
+				   int xioflags, xiofile_t *fd, unsigned groups,
+			   int dummy1, int dummy2, int dummy3);
 
 /* static declaration of ssl's open function */
-static int xioopen_openssl_listen(char *arg, int xioflags, xiofile_t *fd, unsigned groups,
-			   int dummy1, int dummy2, void *dummyp1);
+static int xioopen_openssl_listen(int argc, const char *argv[], struct opt *opts,
+				  int xioflags, xiofile_t *fd, unsigned groups,
+			   int dummy1, int dummy2, int dummy3);
 static int openssl_SSL_ERROR_SSL(int level, const char *funcname);
 static int openssl_handle_peer_certificate(struct single *xfd, bool opt_ver,
 					   int level);
@@ -56,13 +58,13 @@ const struct addrdesc addr_openssl = {
    3,		/* data flow directions this address supports on API layer:
 		   1..read, 2..write, 3..both */
    xioopen_openssl_connect,	/* a function pointer used to "open" these addresses.*/
-   GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_TCP|GROUP_CHILD|GROUP_OPENSSL|GROUP_RETRY,	/* bitwise OR of address groups this address belongs to.
+   GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_IP_TCP|GROUP_CHILD|GROUP_OPENSSL|GROUP_RETRY,	/* bitwise OR of address groups this address belongs to.
 		   You might have to specify a new group in xioopts.h */
    0,		/* an integer passed to xioopen_openssl; makes it possible to
 		   use the same xioopen_openssl function for slightly different
 		   address types. */
    0,		/* like previous argument */
-   NULL		/* like previous arguments, but pointer type.
+   0		/* like previous arguments, but pointer type.
 		   No trailing comma or semicolon! */
    HELP(":<host>:<port>")	/* a text displayed from xio help function.
 			   No trailing comma or semicolon!
@@ -77,13 +79,13 @@ const struct addrdesc addr_openssl_listen = {
    3,		/* data flow directions this address supports on API layer:
 		   1..read, 2..write, 3..both */
    xioopen_openssl_listen,	/* a function pointer used to "open" these addresses.*/
-   GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_TCP|GROUP_LISTEN|GROUP_CHILD|GROUP_RANGE|GROUP_OPENSSL|GROUP_RETRY,	/* bitwise OR of address groups this address belongs to.
+   GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_IP_TCP|GROUP_LISTEN|GROUP_CHILD|GROUP_RANGE|GROUP_OPENSSL|GROUP_RETRY,	/* bitwise OR of address groups this address belongs to.
 		   You might have to specify a new group in xioopts.h */
    0,		/* an integer passed to xioopen_openssl_listen; makes it possible to
 		   use the same xioopen_openssl_listen function for slightly different
 		   address types. */
    0,		/* like previous argument */
-   NULL		/* like previous arguments, but pointer type.
+   0		/* like previous arguments, but pointer type.
 		   No trailing comma or semicolon! */
    HELP(":<port>")	/* a text displayed from xio help function.
 			   No trailing comma or semicolon!
@@ -102,11 +104,41 @@ const struct optdesc opt_openssl_cafile      = { "openssl-cafile",     "cafile",
 const struct optdesc opt_openssl_capath      = { "openssl-capath",     "capath", OPT_OPENSSL_CAPATH,      GROUP_OPENSSL, PH_SPEC, TYPE_FILENAME, OFUNC_SPEC };
 const struct optdesc opt_openssl_egd         = { "openssl-egd",        "egd",    OPT_OPENSSL_EGD,         GROUP_OPENSSL, PH_SPEC, TYPE_FILENAME, OFUNC_SPEC };
 const struct optdesc opt_openssl_pseudo      = { "openssl-pseudo",     "pseudo", OPT_OPENSSL_PSEUDO,      GROUP_OPENSSL, PH_SPEC, TYPE_BOOL,     OFUNC_SPEC };
+#if WITH_FIPS
+const struct optdesc opt_openssl_fips        = { "openssl-fips",       "fips",   OPT_OPENSSL_FIPS,        GROUP_OPENSSL, PH_SPEC, TYPE_BOOL,     OFUNC_SPEC };
+#endif
 
 
+/* If FIPS is compiled in, we need to track if the user asked for FIPS mode.
+ * On forks, the FIPS mode must be reset by a disable, then enable since
+ * FIPS tracks the process ID that initializes things.
+ * If FIPS is not compiled in, no tracking variable is needed
+ * and we make the reset code compile out.  This keeps the
+ * rest of the code below free of FIPS related #ifs
+ */
+#if WITH_FIPS
+static bool xio_openssl_fips = false;
+int xio_reset_fips_mode(void) {
+   if (xio_openssl_fips) {
+      if(!sycFIPS_mode_set(0) || !sycFIPS_mode_set(1)) {
+	 ERR_load_crypto_strings();
+	 ERR_print_errors(BIO_new_fp(stderr,BIO_NOCLOSE));
+	 Error("Failed to reset OpenSSL FIPS mode");
+	 xio_openssl_fips = false;
+         return -1;
+      }
+   }
+   return 0;
+}
+#else
+#define xio_reset_fips_mode() 0
+#endif
+
 /* the open function for OpenSSL client */
 static int
-   xioopen_openssl_connect(char *arg,	/* the arguments in the address string */
+   xioopen_openssl_connect(int argc,
+		   const char *argv[],	/* the arguments in the address string */
+		   struct opt *opts,
 		   int xioflags,	/* is the open meant for reading (0),
 				   writing (1), or both (2) ? */
 		   xiofile_t *xxfd,	/* a xio file descriptor structure,
@@ -116,19 +148,20 @@ static int
 				   addr_openssl */
 		   int dummy2,	/* second transparent integer value from
 				   addr_openssl */
-		   void *dummyp1)	/* transparent pointer value from
+		   int dummy3)	/* transparent pointer value from
 					   addr_openssl */
 {
    struct single *xfd = &xxfd->stream;
-   struct opt *opts = NULL, *opts0 = NULL;
-   char *comma;
-   char *hostname = arg, *portname;
-   const char *protname = "tcp";
+   struct opt *opts0 = NULL;
+   const char *hostname, *portname;
+   int pf = PF_UNSPEC;
    int ipproto = IPPROTO_TCP;
    int socktype = SOCK_STREAM;
    bool dofork = false;
-   struct sockaddr_in us_sa,  *us = &us_sa;
-   struct sockaddr_in themsa, *them = &themsa;
+   union sockaddr_union us_sa,  *us = &us_sa;
+   union sockaddr_union them_sa, *them = &them_sa;
+   socklen_t uslen = sizeof(us_sa);
+   socklen_t themlen = sizeof(them_sa);
    bool needbind = false;
    bool lowport = false;
    int level;
@@ -143,19 +176,12 @@ static int
    }
    xfd->flags |= XIO_DOESCONVERT;
 
-   if (portname = strchr(arg, ':')) {
-      *portname++ = '\0';
-   } else {
-      Error1("xioopen_openssl_connect(\"%s\", ...): missing parameter(s)", arg);
-      return -1;
-   }
-
-   if (comma = strchr(portname, ',')) {
-      *comma++ = '\0';
-   }
-   if (parseopts(comma, groups, &opts) < 0) {
+   if (argc != 3) {
+      Error1("%s: 2 parameters required", argv[0]);
       return STAT_NORETRY;
    }
+   hostname = argv[1];
+   portname = argv[2];
 
    applyopts(-1, opts, PH_INIT);
    if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
@@ -169,8 +195,11 @@ static int
    if (result != STAT_OK)  return STAT_NORETRY;
 
    result =
-      _xioopen_ip4app_prepare(opts, &opts0, hostname, portname, protname,
-			      them, us, &needbind, &lowport, &socktype);
+      _xioopen_ipapp_prepare(opts, &opts0, hostname, portname, &pf, ipproto,
+			     xfd->para.socket.ip.res_opts[1],
+			     xfd->para.socket.ip.res_opts[0],
+			     them, &themlen, us, &uslen,
+			     &needbind, &lowport, &socktype);
    if (result != STAT_OK)  return STAT_NORETRY;
 
    if (xioopts.logopt == 'm') {
@@ -193,8 +222,8 @@ static int
       result =
 	 _xioopen_connect(xfd,
 			  needbind?(struct sockaddr *)us:NULL, sizeof(*us),
-			  (struct sockaddr *)them, sizeof(*them),
-			  opts, PF_INET, socktype, ipproto, lowport, level);
+			  (struct sockaddr *)them, themlen,
+			  opts, pf, socktype, ipproto, lowport, level);
       switch (result) {
       case STAT_OK: break;
 #if WITH_RETRY
@@ -257,12 +286,9 @@ static int
 	 if (pid == 0) {	/* child process */
 	    Info1("just born: OpenSSL client process "F_pid, Getpid());
 
-	    xiodroplocks();	/* drop locks in xio addresses */
-	    /* some locks belong to parent, so "drop" them now */
-	    if (xiohook_newchild) {
-	       if ((*xiohook_newchild)(pid) != 0) {
-		  Exit(1);
-	       }
+	    /* drop parents locks, reset FIPS... */
+	    if (xio_forked_inchild() != 0) {
+	       Exit(1);
 	    }
 	    xfd->forever = false;
 	    xfd->retry = 0;
@@ -340,7 +366,9 @@ int _xioopen_openssl_connect(struct single *xfd,
 #if WITH_LISTEN
 
 static int
-   xioopen_openssl_listen(char *arg,	/* the arguments in the address string */
+   xioopen_openssl_listen(int argc,
+		   const char *argv[],	/* the arguments in the address string */
+		   struct opt *opts,
 		   int xioflags,	/* is the open meant for reading (0),
 				   writing (1), or both (2) ? */
 		   xiofile_t *xxfd,	/* a xio file descriptor structure,
@@ -350,14 +378,17 @@ static int
 				   addr_openssl */
 		   int dummy2,	/* second transparent integer value from
 				   addr_openssl */
-		   void *dummyp1)	/* transparent pointer value from
+		   int dummy3)	/* transparent pointer value from
 					   addr_openssl */
 {
    struct single *xfd = &xxfd->stream;
-   char *a2;
-   struct opt *opts = NULL, *opts0 = NULL;
-   struct sockaddr_in us_sa, *us = &us_sa;
+   const char *portname;
+   struct opt *opts0 = NULL;
+   union sockaddr_union us_sa, *us = &us_sa;
+   socklen_t uslen = sizeof(us_sa);
+   int pf;
    int socktype = SOCK_STREAM;
+   int ipproto = IPPROTO_TCP;
    /*! lowport? */
    int level;
    SSL_CTX* ctx;
@@ -371,14 +402,21 @@ static int
    }
    xfd->flags |= XIO_DOESCONVERT;
 
-   if (a2 = strchr(arg, ',')) {
-      *a2++ = '\0';
-   }
-
-   if (parseopts(a2, groups, &opts) < 0) {
+   if (argc != 2) {
+      Error1("%s: 1 parameter required", argv[0]);
       return STAT_NORETRY;
    }
 
+#if WITH_IP4 && WITH_IP6
+   pf = xioopts.default_ip=='6'?PF_INET6:PF_INET;
+#elif WITH_IP6
+   pf = PF_INET6;
+#else
+   pf = PF_INET;
+#endif
+   
+   portname = argv[1];
+
    applyopts(-1, opts, PH_INIT);
    if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
 
@@ -387,11 +425,16 @@ static int
       Warn("no certificate given; consider option \"cert\"");
    }
 
+   applyopts(-1, opts, PH_EARLY);
+
    result =
       _xioopen_openssl_prepare(opts, xfd, true, &opt_ver, opt_cert, &ctx);
    if (result != STAT_OK)  return STAT_NORETRY;
 
-   if (_xioopen_ip4app_listen_prepare(opts, &opts0, arg, "tcp", us, &socktype)
+   if (_xioopen_ipapp_listen_prepare(opts, &opts0, portname, &pf, ipproto,
+				     xfd->para.socket.ip.res_opts[1],
+				     xfd->para.socket.ip.res_opts[0],
+				     us, &uslen, &socktype)
        != STAT_OK) {
       return STAT_NORETRY;
    }
@@ -411,8 +454,8 @@ static int
       /* tcp listen; this can fork() for us; it only returns on error or on
 	 successful establishment of tcp connection */
       result = _xioopen_listen(xfd, xioflags,
-			       (struct sockaddr *)us, sizeof(*us),
-			       opts, PF_INET, socktype, IPPROTO_TCP,
+			       (struct sockaddr *)us, uslen,
+			       opts, pf, socktype, IPPROTO_TCP,
 #if WITH_RETRY
 			       (xfd->retry||xfd->forever)?E_INFO:E_ERROR
 #else
@@ -439,7 +482,7 @@ static int
       default:
 	 return result;
       }
-      
+
       result = _xioopen_openssl_listen(xfd, opt_ver, ctx, level);
       switch (result) {
       case STAT_OK: break;
@@ -576,6 +619,7 @@ int
 			    const char *opt_cert,
 			    SSL_CTX **ctx)
 {
+   bool opt_fips = false;
    SSL_METHOD *method;
    char *me_str = NULL;	/* method string */
    char *ci_str = NULL;	/* cipher string */
@@ -591,6 +635,7 @@ int
    xfd->addr  = &addr_openssl;
    xfd->dtype = DATA_OPENSSL;
 
+   retropt_bool(opts, OPT_OPENSSL_FIPS, &opt_fips);
    retropt_string(opts, OPT_OPENSSL_METHOD, &me_str);
    retropt_string(opts, OPT_OPENSSL_CIPHERLIST, &ci_str);
    retropt_bool(opts, OPT_OPENSSL_VERIFY, opt_ver);
@@ -601,6 +646,18 @@ int
    retropt_string(opts, OPT_OPENSSL_EGD, &opt_egd);
    retropt_bool(opts,OPT_OPENSSL_PSEUDO, &opt_pseudo);
 
+#if WITH_FIPS
+   if (opt_fips) {
+      if (!sycFIPS_mode_set(1)) {
+	 ERR_load_crypto_strings();
+	 ERR_print_errors(BIO_new_fp(stderr,BIO_NOCLOSE));
+	 Error("Failed to set FIPS mode");
+      } else {
+	 xio_openssl_fips = true;
+      }
+   }
+#endif
+
    OpenSSL_add_all_algorithms();
    OpenSSL_add_all_ciphers();
    OpenSSL_add_all_digests();
@@ -780,7 +837,7 @@ static int openssl_SSL_ERROR_SSL(int level, const char *funcname) {
    if (e == ((ERR_LIB_RAND<<24)|
 	     (RAND_F_SSLEAY_RAND_BYTES<<12)|
 	     (RAND_R_PRNG_NOT_SEEDED)) /*0x24064064*/) {
-      Error("to few entropy; use options \"egd\" or \"pseudo\"");
+      Error("too few entropy; use options \"egd\" or \"pseudo\"");
       return STAT_NORETRY;
    } else {
       Msg2(level, "%s(): %s", funcname, ERR_error_string(e, buf));
