@@ -1,5 +1,5 @@
-/* $Id: xio-openssl.h,v 1.6 2005/03/12 17:37:58 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2002-2005 */
+/* $Id: xio-openssl.h,v 1.7 2006/06/30 20:52:00 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2002-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_openssl_included
@@ -23,6 +23,9 @@ extern const struct optdesc opt_openssl_cafile;
 extern const struct optdesc opt_openssl_capath;
 extern const struct optdesc opt_openssl_egd;
 extern const struct optdesc opt_openssl_pseudo;
+#if WITH_FIPS
+extern const struct optdesc opt_openssl_fips;
+#endif
 
 extern int
    _xioopen_openssl_prepare(struct opt *opts, struct single *xfd,
@@ -39,6 +42,10 @@ extern int xioshutdown_openssl(xiofile_t *xfd, int how);
 extern ssize_t xioread_openssl(struct single *file, void *buff, size_t bufsiz);
 extern ssize_t xiowrite_openssl(struct single *file, const void *buff, size_t bufsiz);
 
+#if WITH_FIPS
+extern int xio_reset_fips_mode(void);
+#endif /* WITH_FIPS */
+
 #endif /* WITH_OPENSSL */
 
 #endif /* !defined(__xio_openssl_included) */
