@@ -1,5 +1,5 @@
-/* $Id: xio-progcall.c,v 1.46 2005/08/18 19:51:25 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: xio-progcall.c,v 1.50 2006/06/30 20:35:11 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains common code dealing with program calls (exec, system) */
@@ -41,8 +41,7 @@ const struct optdesc opt_sigquit = { "sigquit",   NULL, OPT_SIGQUIT,     GROUP_P
    return>0: is parent process
    return<0: error occurred, assume parent process and no child exists !!!
  */
-int _xioopen_foxec(char *comma,
-		int xioflags,	/* XIO_RDONLY etc. */
+int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 		struct single *fd,
 		unsigned groups,
 		   struct opt **copts	/* in: opts; out: opts for child */
@@ -393,18 +392,24 @@ int _xioopen_foxec(char *comma,
    }
 
    if (withfork) {
+      const char *forkwaitstring;
+      int forkwaitsecs = 0;
+
       pid = Fork();
       if (pid < 0) {
 	 Error1("fork(): %s", strerror(errno));
 	 return STAT_RETRYLATER;
       }
+      /* gdb recommends to have env controlled sleep after fork */
+      if (forkwaitstring = getenv("SOCAT_FORK_WAIT")) {
+         forkwaitsecs = atoi(forkwaitstring);
+         Sleep(forkwaitsecs);
+      }
+
       if (pid == 0) {	/* child */
-	 xiodroplocks();	/* drop locks in xio addresses */
-	 /* some locks belong to parent, so "drop" them now */
-	 if (xiohook_newchild) {
-	    if ((*xiohook_newchild)(pid) != 0) {
-	       Exit(1);
-	    }
+	 /* drop parents locks, reset FIPS... */
+	 if (xio_forked_inchild() != 0) {
+	    Exit(1);
 	 }
       }
    }
@@ -533,8 +538,8 @@ int _xioopen_foxec(char *comma,
 	 } else
 #endif /* HAVE_PTY */
 	 if (usepipes) {
-	    if (Dup2(rdpip[1], 2) < 0) {
-	       Error2("dup2(%d, 2): %s", rdpip[1], strerror(errno));
+	    if (Dup2(/*rdpip[1]*/ fdo, 2) < 0) {
+	       Error2("dup2(%d, 2): %s", /*rdpip[1]*/ fdo, strerror(errno));
 	    }
 	    /*0 Info1("dup2(%d, 2)", rdpip[1]);*/
 	 } else {
@@ -567,7 +572,7 @@ int _xioopen_foxec(char *comma,
 #if HAVE_PTY
    if (usepty) {
       if (Close(ttyfd) < 0) {
-	 Warn2("close(%d): %s", ttyfd, strerror(errno));
+	 Info2("close(%d): %s", ttyfd, strerror(errno));
       }
    } /*0 else*/
 #endif /* HAVE_PTY */
