@@ -1,33 +1,35 @@
-/* $Id: xio-proxy.c,v 1.19 2005/08/18 19:51:25 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2002-2005 */
+/* $Id: xio-proxy.c,v 1.26 2006/07/13 06:46:19 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2002-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses of HTTP proxy CONNECT
    type */
 
 #include "xiosysincludes.h"
-#include "xioopen.h"
-#include "xio-socket.h"
-#include "xio-ipapp.h"
-
-#include "xio-ascii.h"	/* for base64 encoding of authentication */
-#include "xio-proxy.h"
-
 
 #if WITH_PROXY
 
+#include "xioopen.h"
+#include "xio-socket.h"
+#include "xio-ipapp.h"
+#include "xio-ascii.h"	/* for base64 encoding of authentication */
+
+#include "xio-proxy.h"
+
+
 #define PROXYPORT "8080"
 
-static int xioopen_proxy_connect(char *a1, int xioflags, xiofile_t *fd,
+static int xioopen_proxy_connect(int argc, const char *argv[], struct opt *opts,
+				 int xioflags, xiofile_t *fd,
 				 unsigned groups, int dummy1, int dummy2,
-				 void *dummyp1);
+				 int dummy3);
 
 const struct optdesc opt_proxyport = { "proxyport", NULL, OPT_PROXYPORT, GROUP_HTTP, PH_LATE, TYPE_STRING, OFUNC_SPEC };
 const struct optdesc opt_ignorecr  = { "ignorecr",  NULL, OPT_IGNORECR,  GROUP_HTTP, PH_LATE, TYPE_BOOL,  OFUNC_SPEC };
 const struct optdesc opt_proxy_resolve   = { "proxy-resolve",   "resolve", OPT_PROXY_RESOLVE,   GROUP_HTTP, PH_LATE, TYPE_BOOL,  OFUNC_SPEC };
 const struct optdesc opt_proxy_authorization  = { "proxy-authorization",  "proxyauth", OPT_PROXY_AUTHORIZATION,  GROUP_HTTP, PH_LATE, TYPE_STRING,  OFUNC_SPEC };
 
-const struct addrdesc addr_proxy_connect = { "proxy", 3, xioopen_proxy_connect, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_TCP|GROUP_HTTP|GROUP_CHILD|GROUP_RETRY, 0, 0, NULL HELP(":<proxy-server>:<host>:<port>") };
+const struct addrdesc addr_proxy_connect = { "proxy", 3, xioopen_proxy_connect, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_IP_TCP|GROUP_HTTP|GROUP_CHILD|GROUP_RETRY, 0, 0, 0 HELP(":<proxy-server>:<host>:<port>") };
 
 
 /*0#define CONNLEN 40*/	/* "CONNECT 123.156.189.123:65432 HTTP/1.0\r\n\0" */
@@ -73,49 +75,38 @@ static ssize_t
 #define BUFLEN 2048
 
 
-static int xioopen_proxy_connect(char *a1, int xioflags, xiofile_t *xxfd,
+static int xioopen_proxy_connect(int argc, const char *argv[], struct opt *opts,
+				 int xioflags, xiofile_t *xxfd,
 				 unsigned groups, int dummy1, int dummy2,
-				 void *dummyp1) {
+				 int dummy3) {
       /* we expect the form: host:host:port */
-   char *a2, *a3, *a4;
    struct single *xfd = &xxfd->stream;
-   struct opt *opts = NULL, *opts0 = NULL;
+   struct opt *opts0 = NULL;
    struct proxyvars struct_proxyvars = { 0 }, *proxyvars = &struct_proxyvars;
    /* variables to be filled with address option values */
    bool dofork = false;
    /* */
-   struct sockaddr_in us_sa,   *us = &us_sa;
-   struct sockaddr_in them_sa, *them = &them_sa;
-   char *proxyname, *proxyport = NULL;
-   char *targetname;
-   const char *protname = "tcp";
+   int pf = PF_UNSPEC;
+   union sockaddr_union us_sa,   *us = &us_sa;
+   union sockaddr_union them_sa, *them = &them_sa;
+   socklen_t uslen = sizeof(us_sa);
+   socklen_t themlen = sizeof(them_sa);
+   const char *proxyname; char *proxyport = NULL;
+   const char *targetname, *targetport;
+   int ipproto = IPPROTO_TCP;
    bool needbind = false;
    bool lowport = false;
    int socktype = SOCK_STREAM;
    int level;
    int result;
 
-   proxyname = a1;
-   a2 = strchr(a1, ':');
-   if (!a2) {
-      Error1("xioopen_proxy_connect(\"%s\",,): missing target address", a1);
-      return STAT_NORETRY;
-   }
-   *a2 = '\0', ++a2;
-   targetname = a2;
-
-   if ((a3 = strchr(a2, ':')) == NULL) {
-      Error2("xioopen_proxy_connect(\"%s:%s\",,): missing target port", proxyname, a2);
-      return STAT_NORETRY;
-   }
-   *a3 = '\0', ++a3;
-
-   a4 = strchr(a3, ',');
-   if (a4)  *a4 = '\0', ++a4;
-
-   if (parseopts(a4, groups, &opts) < 0) {
+   if (argc != 4) {
+      Error1("%s: 3 parameters required", argv[0]);
       return STAT_NORETRY;
    }
+   proxyname = argv[1];
+   targetname = argv[2];
+   targetport = argv[3];
 
    applyopts(-1, opts, PH_INIT);
    if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
@@ -130,12 +121,16 @@ static int xioopen_proxy_connect(char *a1, int xioflags, xiofile_t *xxfd,
       }
    }
 
-   result = _xioopen_proxy_prepare(proxyvars, opts, targetname, a3);
+   result = _xioopen_proxy_prepare(proxyvars, opts, targetname, targetport);
    if (result != STAT_OK)  return result;
 
-   result = _xioopen_ip4app_prepare(opts, &opts0, proxyname, proxyport,
-				    protname, them, us,
-				    &needbind, &lowport, &socktype);
+   result =
+      _xioopen_ipapp_prepare(opts, &opts0, proxyname, proxyport,
+			     &pf, ipproto,
+			     xfd->para.socket.ip.res_opts[1],
+			     xfd->para.socket.ip.res_opts[0],
+			     them, &themlen, us, &uslen,
+			     &needbind, &lowport, &socktype);
    if (result != STAT_OK)  return result;
 
    Notice4("opening connection to %s:%u via proxy %s:%s",
@@ -153,8 +148,8 @@ static int xioopen_proxy_connect(char *a1, int xioflags, xiofile_t *xxfd,
    result =
       _xioopen_connect(xfd,
 		       needbind?(struct sockaddr *)us:NULL, sizeof(*us),
-		       (struct sockaddr *)them, sizeof(*them),
-		       opts, PF_INET, socktype, IPPROTO_TCP, lowport, level);
+		       (struct sockaddr *)them, themlen,
+		       opts, pf, socktype, IPPROTO_TCP, lowport, level);
       switch (result) {
       case STAT_OK: break;
 #if WITH_RETRY
@@ -207,12 +202,9 @@ static int xioopen_proxy_connect(char *a1, int xioflags, xiofile_t *xxfd,
 	 if (pid == 0) {	/* child process */
 	    Info1("just born: proxy client process "F_pid, Getpid());
 
-	    xiodroplocks();	/* drop locks in xio addresses */
-	    /* some locks belong to parent, so "drop" them now */
-	    if (xiohook_newchild) {
-	       if ((*xiohook_newchild)(pid) != 0) {
-		  Exit(1);
-	       }
+	    /* drop parents locks, reset FIPS... */
+	    if (xio_forked_inchild() != 0) {
+	       Exit(1);
 	    }
 	    xfd->forever = false;  xfd->retry = 0;
 	    break;
@@ -271,7 +263,7 @@ int _xioopen_proxy_prepare(struct proxyvars *proxyvars, struct opt *opts,
       proxyvars->targetaddr = strdup(targetname);
    }
 
-   proxyvars->targetport = htons(parseport(targetport, "tcp"));
+   proxyvars->targetport = htons(parseport(targetport, IPPROTO_TCP));
 
    return STAT_OK;
 }
@@ -305,7 +297,7 @@ int _xioopen_proxy_connect(struct single *xfd,
       Msg4(level, "write(%d, %p, "F_Zu"): %s",
 	   xfd->fd, request, strlen(request), strerror(errno));
       if (Close(xfd->fd) < 0) {
-	 Warn2("close(%d): %s", xfd->fd, strerror(errno));
+	 Info2("close(%d): %s", xfd->fd, strerror(errno));
       }
       return STAT_RETRYLATER;
    }
@@ -338,7 +330,7 @@ int _xioopen_proxy_connect(struct single *xfd,
 	 Msg4(level, "write(%d, %p, "F_Zu"): %s",
 	      xfd->fd, header, strlen(header), strerror(errno));
 	 if (Close(xfd->fd) < 0) {
-	    Warn2("close(%d): %s", xfd->fd, strerror(errno));
+	    Info2("close(%d): %s", xfd->fd, strerror(errno));
 	 }
 	 return STAT_RETRYLATER;
       }
