@@ -1,5 +1,5 @@
-/* $Id: xio-pty.c,v 1.15 2005/09/03 16:51:36 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2002-2005 */
+/* $Id: xio-pty.c,v 1.18 2006/05/31 19:16:02 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2002-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for creating pty addresses */
@@ -18,9 +18,9 @@
 
 #define MAXPTYNAMELEN 64
 
-static int xioopen_pty(char *a1, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, void *dummyp1);
+static int xioopen_pty(int argc, const char *argv[], struct opt *opts, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, int dummy3);
 
-const struct addrdesc addr_pty = { "pty",   3, xioopen_pty, GROUP_NAMED|GROUP_FD|GROUP_TERMIOS|GROUP_PTY, 0, 0, NULL HELP("") };
+const struct addrdesc addr_pty = { "pty",   3, xioopen_pty, GROUP_NAMED|GROUP_FD|GROUP_TERMIOS|GROUP_PTY, 0, 0, 0 HELP("") };
 
 const struct optdesc opt_symbolic_link = { "symbolic-link", "link", OPT_SYMBOLIC_LINK, GROUP_PTY, PH_LATE, TYPE_FILENAME, OFUNC_SPEC, 0, 0 };
 #if HAVE_POLL
@@ -28,9 +28,8 @@ const struct optdesc opt_pty_wait_slave = { "pty-wait-slave", "wait-slave", OPT_
 const struct optdesc opt_pty_intervall  = { "pty-intervall",  NULL,         OPT_PTY_INTERVALL,  GROUP_PTY, PH_EARLY, TYPE_TIMESPEC, OFUNC_SPEC, 0, 0 };
 #endif /* HAVE_POLL */
 
-static int xioopen_pty(char *a1, int xioflags, xiofile_t *xfd, unsigned groups, int dummy1, int dummy2, void *dummyp1) {
+static int xioopen_pty(int argc, const char *argv[], struct opt *opts, int xioflags, xiofile_t *xfd, unsigned groups, int dummy1, int dummy2, int dummy3) {
    /* we expect the form: filename */
-   struct opt *opts = NULL;
    int ptyfd = -1, ttyfd = -1;
 #if defined(HAVE_DEV_PTMX) || defined(HAVE_DEV_PTC)
    bool useptmx = false;	/* use /dev/ptmx or equivalent */
@@ -48,8 +47,6 @@ static int xioopen_pty(char *a1, int xioflags, xiofile_t *xfd, unsigned groups,
 				   compatibility we choose "no" as default */
    struct timespec pollintv = { PTY_INTERVALL };
 
-   if (parseopts(a1, groups, &opts) < 0)
-      return STAT_NORETRY;
    applyopts(-1, opts, PH_INIT);
    if (applyopts_single(&xfd->stream, opts, PH_INIT) < 0)  return -1;
 
