@@ -1,5 +1,5 @@
-/* $Id: xio-readline.c,v 1.11 2005/03/13 12:17:26 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2002-2005 */
+/* $Id: xio-readline.c,v 1.15 2006/03/21 19:15:13 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2002-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening the readline address */
@@ -25,26 +25,32 @@ uses stdin!!
 /* length of buffer for dynamic prompt */
 #define READLINE_MAXPROMPT 512
 
-static int xioopen_readline(char *arg, int rw, xiofile_t *xfd, unsigned groups,
-			    int dummy1, int dummy2, void *dummyp1);
+static int xioopen_readline(int argc, const char *argv[], struct opt *opts,
+			    int rw, xiofile_t *xfd, unsigned groups,
+			    int dummy1, int dummy2, int dummy3);
 
 
 const struct addrdesc addr_readline = {
-   "readline", 3, xioopen_readline, GROUP_FD|GROUP_TERMIOS|GROUP_READLINE, 0, 0, NULL HELP(NULL) };
+   "readline", 3, xioopen_readline, GROUP_FD|GROUP_TERMIOS|GROUP_READLINE, 0, 0, 0 HELP(NULL) };
 
 const struct optdesc opt_history_file = { "history-file", "history", OPT_HISTORY_FILE, GROUP_READLINE, PH_LATE, TYPE_STRING, OFUNC_OFFSET, (int)&((xiofile_t *)0)->stream.para.readline.history_file };
 const struct optdesc opt_prompt       = { "prompt",       NULL,      OPT_PROMPT,       GROUP_READLINE, PH_LATE, TYPE_STRING, OFUNC_OFFSET, (int)&((xiofile_t *)0)->stream.para.readline.prompt };
 const struct optdesc opt_noprompt     = { "noprompt",     NULL,      OPT_NOPROMPT,     GROUP_READLINE, PH_LATE, TYPE_BOOL,   OFUNC_SPEC,   0 };
 const struct optdesc opt_noecho       = { "noecho",       NULL,      OPT_NOECHO,       GROUP_READLINE, PH_LATE, TYPE_STRING, OFUNC_SPEC,   0 };
 
-static int xioopen_readline(char *arg, int xioflags, xiofile_t *xfd, unsigned groups,
-			    int dummy1, int dummy2, void *dummyp1) {
+static int xioopen_readline(int argc, const char *argv[], struct opt *opts,
+			    int xioflags, xiofile_t *xfd, unsigned groups,
+			    int dummy1, int dummy2, int dummy3) {
    int rw = (xioflags & XIO_ACCMODE);
    char msgbuf[256], *cp = msgbuf;
-   struct opt *opts = NULL;
    bool noprompt = false;
    char *noecho = NULL;
 
+   if (argc != 1) {
+      Error1("%s: 0 parameters required", argv[0]);
+      return STAT_NORETRY;
+   }
+
    if (!(xioflags & XIO_MAYCONVERT)) {
       Error("address with data processing not allowed here");
       return STAT_NORETRY;
@@ -63,13 +69,6 @@ static int xioopen_readline(char *arg, int xioflags, xiofile_t *xfd, unsigned gr
    }
    Notice(msgbuf);
 
-#if 0
-   if (*arg != '\0' && *arg != ',') {
-      Error("readline takes no parameters");
-      return -1;
-   }
-#endif
-
    xfd->stream.fd = 0;	/* stdin */
    xfd->stream.howtoend = END_NONE;
    xfd->stream.dtype    = DATA_READLINE;
@@ -85,9 +84,8 @@ static int xioopen_readline(char *arg, int xioflags, xiofile_t *xfd, unsigned gr
    }
 #endif /* WITH_TERMIOS */
 
-   if (parseopts(arg, groups|groupbits(xfd->stream.fd), &opts) < 0)
-      return STAT_NORETRY;
    applyopts(-1, opts, PH_INIT);
+   if (applyopts_single(&xfd->stream, opts, PH_INIT) < 0)  return -1;
 
    applyopts2(xfd->stream.fd, opts, PH_INIT, PH_FD);
 
@@ -131,20 +129,32 @@ static int xioopen_readline(char *arg, int xioflags, xiofile_t *xfd, unsigned gr
 ssize_t xioread_readline(struct single *pipe, void *buff, size_t bufsiz) {
    /*! indent */
    ssize_t bytes;
-      char *line;
+   char *line;
+   int _errno;
 
 #if HAVE_REGEX_H
       if (pipe->para.readline.dynprompt &&
 	  pipe->para.readline.hasnoecho &&
 	  !regexec(&pipe->para.readline.noecho,
 		   pipe->para.readline.dynprompt, 0, NULL, 0)) {
+      /* under these conditions, we do not echo input, thus we circumvent
+	 readline */
 	 struct termios saveterm, setterm;
 	 *pipe->para.readline.dynend = '\0';
 	 Tcgetattr(pipe->fd, &saveterm);	/*! error */
 	 setterm = saveterm;
 	 setterm.c_lflag |= ICANON;
 	 Tcsetattr(pipe->fd, TCSANOW, &setterm);	/*!*/
-	 bytes = Read(pipe->fd, buff, bufsiz);	/*!*/
+	 do {
+	    bytes = Read(pipe->fd, buff, bufsiz);
+	 } while (bytes < 0 && errno == EINTR);
+	 if (bytes < 0) {
+	    _errno = errno;
+	    Error4("read(%d, %p, "F_Zu"): %s",
+		   pipe->fd, buff, bufsiz, strerror(_errno));
+	    errno = _errno;
+	    return -1;
+	 }
 	 setterm.c_lflag &= ~ICANON;
 	 Tcgetattr(pipe->fd, &setterm);	/*! error */
 	 Tcsetattr(pipe->fd, TCSANOW, &saveterm);	/*!*/
@@ -177,6 +187,7 @@ ssize_t xioread_readline(struct single *pipe, void *buff, size_t bufsiz) {
       } else {
 	 line = Readline(pipe->para.readline.prompt);
       }
+      /* GNU readline defines no error return */
       if (line == NULL) {
 	 return 0;	/* EOF */
       }
