@@ -1,5 +1,5 @@
-/* $Id: xio-socket.h,v 1.11 2005/03/18 20:25:38 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: xio-socket.h,v 1.15 2006/05/31 19:24:36 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_socket_h_included
@@ -49,6 +49,9 @@ extern const struct optdesc opt_so_prototype;
 extern const struct optdesc opt_fiosetown;
 extern const struct optdesc opt_siocspgrp;
 extern const struct optdesc opt_bind;
+extern const struct optdesc opt_protocol_family;
+
+extern int retropt_socket_pf(struct opt *opts, int *pf);
 
 extern int xioopen_connect(struct single *fd,
 			    struct sockaddr *us, size_t uslen,
@@ -61,4 +64,27 @@ extern int _xioopen_connect(struct single *fd,
 			    struct opt *opts, int pf, int stype, int proto,
 			    bool alt, int level);
 
+/* common to xioopen_udp_sendto, ..unix_sendto, ..rawip */
+extern 
+int _xioopen_dgram_sendto(union sockaddr_union *them, socklen_t themlen,
+			union sockaddr_union *us, socklen_t uslen,
+			struct opt *opts,
+			int xioflags, xiosingle_t *xfd, unsigned groups,
+			int pf, int socktype, int ipproto);
+extern
+int _xioopen_dgram_recvfrom(struct single *xfd, int xioflags,
+			    struct sockaddr *us, socklen_t uslen,
+			    struct opt *opts,
+			    int pf, int socktype, int proto, int level);
+extern
+int _xioopen_dgram_recv(struct single *xfd, int xioflags,
+			struct sockaddr *us, socklen_t uslen,
+			struct opt *opts, int pf, int socktype, int proto,
+			int level);
+extern 
+int xiogetpacketsrc(int fd, union sockaddr_union *pa, socklen_t *palen);
+extern
+int xiocheckpeer(xiosingle_t *xfd,
+		 union sockaddr_union *pa, union sockaddr_union *la);
+
 #endif /* !defined(__xio_socket_h_included) */
