@@ -1,18 +1,21 @@
-/* $Id: xio-socks.c,v 1.24 2005/08/18 19:51:25 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: xio-socks.c,v 1.31 2006/07/13 06:48:19 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses of socks4 type */
 
 #include "xiosysincludes.h"
+
+#if WITH_SOCKS4 || WITH_SOCKS4A
+
 #include "xioopen.h"
 #include "xio-socket.h"
+#include "xio-ip.h"
 #include "xio-ipapp.h"
 
 #include "xio-socks.h"
 
 
-#if WITH_SOCKS4 || WITH_SOCKS4A
 enum {
    SOCKS_CD_GRANTED = 90,
    SOCKS_CD_FAILED,
@@ -23,30 +26,34 @@ enum {
 #define SOCKSPORT "1080"
 #define BUFF_LEN (SIZEOF_STRUCT_SOCKS4+512)
 
-static int xioopen_socks4_connect(char *a1, int xioflags, xiofile_t *fd,
+static int xioopen_socks4_connect(int argc, const char *argv[], struct opt *opts,
+				  int xioflags, xiofile_t *fd,
 				  unsigned groups, int dummy1, int dummy2,
-				  void *dummyp1);
+				  int dummy3);
 
 const struct optdesc opt_socksport = { "socksport", NULL, OPT_SOCKSPORT, GROUP_IP_SOCKS4, PH_LATE, TYPE_STRING, OFUNC_SPEC };
 const struct optdesc opt_socksuser = { "socksuser", NULL, OPT_SOCKSUSER, GROUP_IP_SOCKS4, PH_LATE, TYPE_NAME, OFUNC_SPEC };
 
-const struct addrdesc addr_socks4_connect = { "socks4", 3, xioopen_socks4_connect, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_TCP|GROUP_IP_SOCKS4|GROUP_CHILD|GROUP_RETRY, 0, 0, NULL HELP(":<socks-server>:<host>:<port>") };
+const struct addrdesc addr_socks4_connect = { "socks4", 3, xioopen_socks4_connect, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_IP_TCP|GROUP_IP_SOCKS4|GROUP_CHILD|GROUP_RETRY, 0, 0, 0 HELP(":<socks-server>:<host>:<port>") };
 
-const struct addrdesc addr_socks4a_connect = { "socks4a", 3, xioopen_socks4_connect, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_TCP|GROUP_IP_SOCKS4|GROUP_CHILD|GROUP_RETRY, 1, 0, NULL HELP(":<socks-server>:<host>:<port>") };
+const struct addrdesc addr_socks4a_connect = { "socks4a", 3, xioopen_socks4_connect, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_IP_TCP|GROUP_IP_SOCKS4|GROUP_CHILD|GROUP_RETRY, 1, 0, 0 HELP(":<socks-server>:<host>:<port>") };
 
-static int xioopen_socks4_connect(char *a1, int xioflags, xiofile_t *xxfd,
+static int xioopen_socks4_connect(int argc, const char *argv[], struct opt *opts,
+				  int xioflags, xiofile_t *xxfd,
 				  unsigned groups, int socks4a, int dummy2,
-				  void *dummyp1) {
+				  int dummy3) {
    /* we expect the form: host:host:port */
-   char *a2, *a3, *a4;
    struct single *xfd = &xxfd->stream;
-   struct opt *opts = NULL, *opts0 = NULL;
-   char *sockdname = a1, *socksport = NULL;
-   const char *targetname;
-   const char *protname = "tcp";
+   struct opt *opts0 = NULL;
+   const char *sockdname; char *socksport;
+   const char *targetname, *targetport;
+   int pf = PF_UNSPEC;
+   int ipproto = IPPROTO_TCP;
    bool dofork = false;
-   struct sockaddr_in us_sa,  *us = &us_sa;
-   struct sockaddr_in themsa, *them = &themsa;
+   union sockaddr_union us_sa,  *us = &us_sa;
+   union sockaddr_union them_sa, *them = &them_sa;
+   socklen_t uslen = sizeof(us_sa);
+   socklen_t themlen = sizeof(them_sa);
    bool needbind = false;
    bool lowport = false;
    unsigned char buff[BUFF_LEN];
@@ -56,28 +63,13 @@ static int xioopen_socks4_connect(char *a1, int xioflags, xiofile_t *xxfd,
    int level;
    int result;
 
-   a2 = strchr(a1, ':');
-   if (!a2) {
-      Error1("xioopen_socks4_connect(\"%s\",,): missing target address", a1);
+   if (argc != 4) {
+      Error1("%s: 3 parameters required", argv[0]);
       return STAT_NORETRY;
    }
-   *a2 = '\0', ++a2;
-   targetname = a2;
-
-   if ((a3 = strchr(a2, ':')) == NULL) {
-      Error2("xioopen_socks4_connect(\"%s:%s\",,): missing target port", a1, a2);
-      return STAT_NORETRY;
-   }
-   *a3 = '\0', ++a3;
-
-   a4 = strchr(a3, ',');
-   if (a4)  *a4 = '\0', ++a4;
-
-   if (parseopts(a4, groups, &opts) < 0) {
-      return STAT_NORETRY;
-   }
-
-   socket_in_init(us);
+   sockdname = argv[1];
+   targetname = argv[2];
+   targetport = argv[3];
 
    applyopts(-1, opts, PH_INIT);
    if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
@@ -86,18 +78,20 @@ static int xioopen_socks4_connect(char *a1, int xioflags, xiofile_t *xxfd,
 
    retropt_bool(opts, OPT_FORK, &dofork);
 
-   result = _xioopen_socks4_prepare(a3, opts, &socksport, sockhead, &buflen);
-   if (result != STAT_OK)  return result;
-
-   result = _xioopen_ip4app_prepare(opts, &opts0, sockdname, socksport,
-				    protname, them, us,
-				    &needbind, &lowport, &socktype);
+   result = _xioopen_socks4_prepare(targetport, opts, &socksport, sockhead, &buflen);
    if (result != STAT_OK)  return result;
+   result =
+      _xioopen_ipapp_prepare(opts, &opts0, sockdname, socksport,
+			     &pf, ipproto,
+			     xfd->para.socket.ip.res_opts[1],
+			     xfd->para.socket.ip.res_opts[0],
+			     them, &themlen, us, &uslen,
+			     &needbind, &lowport, &socktype);
 
    Notice5("opening connection to %s:%u via socks4 server %s:%s as user \"%s\"",
-	   a2,
+	   targetname,
 	   ntohs(sockhead->port),
-	   a1, socksport, sockhead->userid);
+	   sockdname, socksport, sockhead->userid);
 
    do {	/* loop over failed connect and socks-request attempts */
 
@@ -131,8 +125,8 @@ static int xioopen_socks4_connect(char *a1, int xioflags, xiofile_t *xxfd,
       result =
 	 _xioopen_connect (xfd,
 			   needbind?(struct sockaddr *)us:NULL, sizeof(*us),
-			   (struct sockaddr *)them, sizeof(struct sockaddr_in),
-			   opts, PF_INET, socktype, IPPROTO_TCP, lowport, level);
+			   (struct sockaddr *)them, themlen,
+			   opts, pf, socktype, IPPROTO_TCP, lowport, level);
       switch (result) {
       case STAT_OK: break;
 #if WITH_RETRY
@@ -185,12 +179,9 @@ static int xioopen_socks4_connect(char *a1, int xioflags, xiofile_t *xxfd,
 	 if (pid == 0) {	/* child process */
 	    Info1("just born: socks client process "F_pid, Getpid());
 
-	    xiodroplocks();	/* drop locks in xio addresses */
-	    /* some locks belong to parent, so "drop" them now */
-	    if (xiohook_newchild) {
-	       if ((*xiohook_newchild)(pid) != 0) {
-		  Exit(1);
-	       }
+	    /* drop parents locks, reset FIPS... */
+	    if (xio_forked_inchild() != 0) {
+	       Exit(1);
 	    }
 	    xfd->forever = false;  xfd->retry = 0;
 	    break;
@@ -219,7 +210,7 @@ int _xioopen_socks4_prepare(const char *targetport, struct opt *opts, char **soc
    /* generate socks header - points to final target */
    sockhead->version = 4;
    sockhead->action  = 1;
-   sockhead->port    = parseport(targetport, "tcp");
+   sockhead->port    = parseport(targetport, IPPROTO_TCP);
 
    if (retropt_string(opts, OPT_SOCKSPORT, socksport) < 0) {
       if ((se = getservbyname("socks", "tcp")) != NULL) {
@@ -258,17 +249,21 @@ int
 			    ssize_t *headlen,		/* get available space,
 							   return used length*/
 			    int level) {
-   struct hostent *host = NULL;
    int result;
 
    if (!socks4a) {
-      if ((result = xioGethostbyname(hostname, &host, level)) != STAT_OK) {
-	 return result;
-      }
-   }
+      union sockaddr_union sau;
+      socklen_t saulen = sizeof(sau);
 
-   if (!socks4a || host != NULL) {
-      memcpy(&sockhead->dest, host->h_addr_list[0], 4);
+      if ((result = xiogetaddrinfo(hostname, NULL,
+				   PF_INET, SOCK_STREAM, IPPROTO_TCP,
+				   &sau, &saulen,
+				   xfd->para.socket.ip.res_opts[1],
+				   xfd->para.socket.ip.res_opts[0]))
+	  != STAT_OK) {
+	 return result;	/*! STAT_RETRY? */
+      }
+      memcpy(&sockhead->dest, &sau.ip4.sin_addr, 4);
    }
 #if WITH_SOCKS4A
    else {
@@ -277,7 +272,7 @@ int
    }
 #endif /* WITH_SOCKS4A */
 #if WITH_SOCKS4A
-   if (host == NULL) {
+   if (socks4a) {
       /* SOCKS4A requires us to append the host name to resolve
          after the user name's trailing 0 byte.  */
       char* insert_position = (char*) sockhead + *headlen;
@@ -314,7 +309,7 @@ int _xioopen_socks4_connect(struct single *xfd,
       Msg4(level, "write(%d, %p, "F_Zu"): %s",
 	   xfd->fd, sockhead, headlen, strerror(errno));
       if (Close(xfd->fd) < 0) {
-	 Warn2("close(%d): %s", xfd->fd, strerror(errno));
+	 Info2("close(%d): %s", xfd->fd, strerror(errno));
       }
       return STAT_RETRYLATER;	/* retry complete open cycle */
    }
@@ -331,13 +326,13 @@ int _xioopen_socks4_connect(struct single *xfd,
 	      xfd->fd, buff+bytes, SIZEOF_STRUCT_SOCKS4-bytes,
 	      strerror(errno));
 	 if (Close(xfd->fd) < 0) {
-	    Warn2("close(%d): %s", xfd->fd, strerror(errno));
+	    Info2("close(%d): %s", xfd->fd, strerror(errno));
 	 }
       }
       if (result == 0) {
 	 Msg(level, "read(): EOF during read of socks reply");
 	 if (Close(xfd->fd) < 0) {
-	    Warn2("close(%d): %s", xfd->fd, strerror(errno));
+	    Info2("close(%d): %s", xfd->fd, strerror(errno));
 	 }
 	 return STAT_RETRYLATER;
       }
