@@ -1,5 +1,5 @@
-/* $Id: xio-system.c,v 1.11 2005/03/13 12:18:03 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: xio-system.c,v 1.13 2006/03/21 20:53:38 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses of system type */
@@ -13,37 +13,28 @@
 
 #if WITH_SYSTEM
 
-static int xioopen_system(char *arg,
+static int xioopen_system(int arg, const char *argv[], struct opt *opts,
 		int xioflags,	/* XIO_RDONLY etc. */
 		xiofile_t *fd,
 		unsigned groups,
-		int dummy1, int dummy2, void *dummyp1
+		int dummy1, int dummy2, int dummy3
 		);
 
-const struct addrdesc addr_system = { "system", 3, xioopen_system, GROUP_FD|GROUP_FORK|GROUP_EXEC|GROUP_SOCKET|GROUP_SOCK_UNIX|GROUP_TERMIOS|GROUP_FIFO|GROUP_PTY|GROUP_PARENT, 1, 0, NULL HELP(":<shell-command>") };
+const struct addrdesc addr_system = { "system", 3, xioopen_system, GROUP_FD|GROUP_FORK|GROUP_EXEC|GROUP_SOCKET|GROUP_SOCK_UNIX|GROUP_TERMIOS|GROUP_FIFO|GROUP_PTY|GROUP_PARENT, 1, 0, 0 HELP(":<shell-command>") };
 
 
-static int xioopen_system(char *arg,
+static int xioopen_system(int argc, const char *argv[], struct opt *opts,
 		int xioflags,	/* XIO_RDONLY etc. */
 		xiofile_t *fd,
 		unsigned groups,
-		int dummy1, int dummy2, void *dummyp1
+		int dummy1, int dummy2, int dummy3
 		) {
-   char *comma;
    int status;
-   struct opt *opts = NULL;
    char *path = NULL;
    int result;
-   char *string = arg;
+   const char *string = argv[1];
 
-   if (comma = strchr(arg, ',')) {
-      *comma++ = '\0';
-   }
-
-   if (parseopts(comma, groups, &opts) < 0)
-      return STAT_NORETRY;
-
-   status = _xioopen_foxec(comma, xioflags, &fd->stream, groups, &opts);
+   status = _xioopen_foxec(xioflags, &fd->stream, groups, &opts);
    if (status < 0)  return status;
    if (status == 0) {	/* child */
       int numleft;
