@@ -1,32 +1,39 @@
-/* $Id: xio-tcp.c,v 1.20 2005/08/18 19:52:32 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: xio-tcp.c,v 1.24 2006/07/13 06:48:47 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for TCP related functions and options */
 
 #include "xiosysincludes.h"
-#include "xioopen.h"
 
+#if WITH_TCP
+
+#include "xioopen.h"
 #include "xio-listen.h"
 #include "xio-ip4.h"
 #include "xio-ipapp.h"
 #include "xio-tcp.h"
 
-#if WITH_TCP
-
 /****** TCP addresses ******/
 
-#if WITH_IP4
-const struct addrdesc addr_tcp4_connect = { "tcp4", 3 /* O_RDWR */, xioopen_ip4app_connect, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_TCP|GROUP_CHILD|GROUP_RETRY, SOCK_STREAM, IPPROTO_TCP, "tcp" HELP(":<host>:<port>") };
+#if WITH_IP4 || WITH_IP6
+const struct addrdesc addr_tcp_connect = { "tcp-connect", 1+XIO_RDWR, xioopen_ipapp_connect, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_IP_TCP|GROUP_CHILD|GROUP_RETRY, SOCK_STREAM, IPPROTO_TCP, PF_UNSPEC HELP(":<host>:<port>") };
 #if WITH_LISTEN
-const struct addrdesc addr_tcp4_listen  = { "tcp4-listen", 3, xioopen_ip4app_listen, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_TCP|GROUP_LISTEN|GROUP_CHILD|GROUP_RANGE|GROUP_RETRY, SOCK_STREAM, IPPROTO_TCP, "tcp" HELP(":<port>") };
+const struct addrdesc addr_tcp_listen  = { "tcp-listen",  1+XIO_RDWR, xioopen_ipapp_listen,  GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_IP_TCP|GROUP_LISTEN|GROUP_CHILD|GROUP_RANGE|GROUP_RETRY, SOCK_STREAM, IPPROTO_TCP, PF_UNSPEC HELP(":<port>") };
+#endif
+#endif
+
+#if WITH_IP4
+const struct addrdesc addr_tcp4_connect = { "tcp4-connect", 1+XIO_RDWR, xioopen_ipapp_connect, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_TCP|GROUP_CHILD|GROUP_RETRY, SOCK_STREAM, IPPROTO_TCP, PF_INET HELP(":<host>:<port>") };
+#if WITH_LISTEN
+const struct addrdesc addr_tcp4_listen  = { "tcp4-listen", 1+XIO_RDWR, xioopen_ipapp_listen, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_TCP|GROUP_LISTEN|GROUP_CHILD|GROUP_RANGE|GROUP_RETRY, SOCK_STREAM, IPPROTO_TCP, PF_INET HELP(":<port>") };
 #endif
 #endif /* WITH_IP4 */
 
 #if WITH_IP6
-const struct addrdesc addr_tcp6_connect = { "tcp6",    3, xioopen_ip6app_connect, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP6|GROUP_IP_TCP|GROUP_CHILD|GROUP_RETRY, SOCK_STREAM, IPPROTO_TCP, "tcp" HELP(":<host>:<port>") };
+const struct addrdesc addr_tcp6_connect = { "tcp6-connect", 1+XIO_RDWR, xioopen_ipapp_connect, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP6|GROUP_IP_TCP|GROUP_CHILD|GROUP_RETRY, SOCK_STREAM, IPPROTO_TCP, PF_INET6 HELP(":<host>:<port>") };
 #if WITH_LISTEN
-const struct addrdesc addr_tcp6_listen  = { "tcp6-listen", 3, xioopen_tcp6_listen, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP6|GROUP_IP_TCP|GROUP_LISTEN|GROUP_CHILD|GROUP_RANGE|GROUP_RETRY, SOCK_STREAM, IPPROTO_TCP, "tcp" HELP(":<port>") };
+const struct addrdesc addr_tcp6_listen  = { "tcp6-listen", 1+XIO_RDWR, xioopen_ipapp_listen, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP6|GROUP_IP_TCP|GROUP_LISTEN|GROUP_CHILD|GROUP_RANGE|GROUP_RETRY, SOCK_STREAM, IPPROTO_TCP, PF_INET6 HELP(":<port>") };
 #endif
 #endif /* WITH_IP6 */
 
@@ -109,52 +116,4 @@ const struct optdesc opt_tcp_sackena = { "tcp-sackena", "sackena", OPT_TCP_SACKE
 const struct optdesc opt_tcp_tsoptena = { "tcp-tsoptena", "tsoptena", OPT_TCP_TSOPTENA, GROUP_IP_TCP, PH_PASTSOCKET, TYPE_BOOL, OFUNC_SOCKOPT, SOL_TCP, TCP_TSOPTENA };
 #endif
 
-#if WITH_IP6 && WITH_LISTEN
-/* we expect the form: port */
-int xioopen_tcp6_listen(char *a1, int xioflags, xiofile_t *fd,
-			     unsigned groups, int dummy1, int dummy2,
-			     void *dummyp1) {
-   char *a2;
-   struct opt *opts = NULL, *opts0;
-   struct sockaddr_in6 us;
-   struct sockaddr_in6 them;
-   int needbind = 0;
-   int socktype = SOCK_STREAM;
-   int result;
-
-   if (a2 = strchr(a1, ',')) {
-      *a2++ = '\0';
-   }
-
-   /* initialization of local vars */
-   socket_in6_init(&us);
-   socket_in6_init(&them);
-
-   if (parseopts(a2, groups, &opts) < 0) {
-      return STAT_NORETRY;
-   }
-
-   /* destination port */
-   /*0 us.sin6_port       = htons(strtoul(a1, &a1, 0));*/
-   us.sin6_port = parseport(a1, "tcp");
-
-   if (retropt_bind(opts, AF_INET6, "tcp", (struct sockaddr *)&us, 1) >= 0)
-      needbind = true;
-
-   applyopts(-1, opts, PH_INIT);
-   if (applyopts_single(&fd->stream, opts, PH_INIT) < 0)  return -1;
-   retropt_int(opts, OPT_SO_TYPE, &socktype);
-
-   opts0 = copyopts(opts, GROUP_ALL);
-
-   if ((result =
-	xioopen_listen(&fd->stream, xioflags,
-			(struct sockaddr *)&us, sizeof(struct sockaddr_in6),
-		       opts, opts0, PF_INET6, socktype, IPPROTO_TCP))
-       != 0)
-      return result;
-   return 0;
-}
-#endif /* WITH_IP6 && WITH_LISTEN */
-
 #endif /* WITH_TCP */
