@@ -1,5 +1,5 @@
-/* $Id: xio.h,v 1.46 2005/03/13 12:09:09 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: xio.h,v 1.58 2006/07/13 06:51:34 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_h_included
@@ -20,6 +20,7 @@
 #define LINETERM_CRNL 2
 
 struct addrdesc;
+struct opt;
 
 /* the flags argument of xioopen */
 #define XIO_RDONLY  O_RDONLY /* asserted to be 0 */
@@ -49,19 +50,34 @@ enum xiotag {
 			   streams */
 } ;
 
+/* global XIO options/parameters */
 typedef struct {
    bool strictopts;
    const char *pipesep;
+   const char *paramsep;
+   const char *optionsep;
    char ip4portsep;
-   char ip6portsep;	/* do not change, is hardcoded in parseip6addr()! */
+   char ip6portsep;	/* do not change, might be hardcoded somewhere! */
    char logopt;	/* 'm' means "switch to syslog when entering daemon mode" */
    const char *syslogfac;	/* syslog facility (only with mixed mode) */
+   char default_ip;	/* default prot.fam for IP based listen ('4' or '6') */
+   char preferred_ip;	/* preferred prot.fam. for name resolution ('0' for
+			   unspecified, '4', or '6') */
 } xioopts_t;
 
+/* pack the description of a lock file */
+typedef struct {
+   const char     *lockfile;	/* name of lockfile; NULL if no locking */
+   bool            waitlock;	/* dont't exit when already locked */
+   struct timespec intervall;	/* polling intervall */
+} xiolock_t;
+
 extern xioopts_t xioopts;
 
+#define MAXARGV 8
+
 /* a non-dual file descriptor */ 
-struct single {
+typedef struct single {
    enum xiotag tag;	/* see  enum xiotag  */
    const struct addrdesc *addr;
    int    flags;
@@ -76,13 +92,28 @@ struct single {
 			   occurred 
 			   2..fd0 has reached EOF (definitely; never with
 			   ignoreeof! */
-   size_t wsize;	/* write alway this size; 0..all available */
+   size_t wsize;	/* write always this size; 0..all available */
+   size_t readbytes;	/* read only so many bytes; 0...unlimited */
+   size_t actbytes;	/* so many bytes still to be read (when readbytes!=0)*/
+   xiolock_t lock;	/* parameters of lockfile */
+   bool      havelock;	/* we are happy owner of the above lock */
+   bool	     cool_write;	/* downlevel EPIPE, ECONNRESET to notice */
    /* until here, keep consistent with bipipe.dual ! */
+   int argc;		/* number of fields in argv */
+   const char *argv[MAXARGV];	/* address keyword, required args */
+   struct opt *opts;	/* the options of this address */
    int    lineterm;	/* 0..dont touch; 1..CR; 2..CRNL on extern data */
    int    fd;
+   bool   opt_unlink_close;	/* option unlink_close */
+   char  *unlink_close;	/* name of a symlink or unix socket to be removed */
    enum {
       DATA_STREAM,	/* files and connected sockets (use read, write) */
-      DATA_RECV,	/* datagram sockets (use recvfrom, sendto) */
+      DATA_RECVFROM,	/* datagram sockets (use recvfrom, sendto) */
+      DATA_RECVFROM_SKIPIP,	/* datagram sockets (on recvfrom ignore IP header) */
+      DATA_RECVFROM_ONE,	/* datagram sockets, one packet only */
+      DATA_RECVFROM_SKIPIP_ONE,	/* datagram sockets, ignore IP header, one packet */
+      DATA_RECV,	/* receive data on socket from different sources */
+      DATA_RECV_SKIPIP,	/* receive data from different sources, drop IP hdr */
       DATA_PIPE,	/* a short cut pipe */
       DATA_2PIPE,	/* communication with one process via 2 pipes */
       DATA_PTY,		/* an "internal" pty */
@@ -108,13 +139,36 @@ struct single {
 #endif /* WITH_TERMIOS */
    const char *name;		/* only with END_UNLINK */
    int (*sigchild)(struct single *);	/* callback after sigchild */
+   pid_t ppid;			/* parent pid, only if we send it signals */
    union {
       struct {
 	 int fdout;		/* use fd for output */
       } bipipe;
+#if WITH_SOCKET
       struct {
 	 struct timeval connect_timeout; /* how long to hang in connect() */
+	 union sockaddr_union la;	/* local socket address */
+	 char *rangename;
+	 bool emptyiseof;	/* with dgram: empty packet means EOF */
+	 union xiorange_union range;	/* restrictions for peer address */
+#if WITH_IP4 || WITH_IP6
+	 struct {
+	    unsigned int res_opts[2];	/* bits to be set in _res.options are
+				       at [0], bits to be cleared are at [1] */
+	    bool   dosourceport;
+	    uint16_t sourceport;
+	    bool     lowport;
+#if (WITH_TCP || WITH_UDP) && WITH_LIBWRAP
+	    bool   dolibwrap;
+	    char    *libwrapname;
+	    char    *tcpwrap_etc;
+	    char    *hosts_allow_table;
+	    char    *hosts_deny_table;
+#endif
+	 } ip;
+#endif /* WITH_IP4 || WITH_IP6 */
       } socket;
+#endif /* WITH_SOCKET */
       struct {
 	 pid_t pid;		/* child PID, with EXEC: */
 	 int fdout;		/* use fd for output if two pipes */
@@ -140,7 +194,7 @@ struct single {
       } openssl;
 #endif /* WITH_OPENSSL */
    } para;
-} ;
+} xiosingle_t;
 
 /* rw: 0..read, 1..write, 2..r/w */
 /* when implementing a new address type take care of following topics:
@@ -172,7 +226,11 @@ typedef union bipipe {
       bool        ignoreeof;
       int         eof;		/* fd0 has reached EOF */
       size_t      wsize;	/* write always this size; 0..all available */
-      union bipipe *stream[2];	/* input stream, output stream */
+      size_t readbytes;	/* read only so many bytes; 0...unlimited */
+      size_t actbytes;	/* so many bytes still to be read */
+      xiolock_t lock;	/* parameters of lockfile */
+      bool      havelock;	/* we are happy owner of the above lock */
+      xiosingle_t *stream[2];	/* input stream, output stream */
    } dual;
 } xiofile_t;
 
@@ -180,12 +238,12 @@ typedef union bipipe {
 struct addrdesc {
    const char *defname;	/* main (canonical) name of address */
    int directions;	/* 1..read, 2..write, 3..both */
-   int (*func)(char *a1, int rw, xiofile_t *fd, unsigned groups,
-	       int arg1, int arg2, void *argp1);
+   int (*func)(int argc, const char *argv[], struct opt *opts, int rw, xiofile_t *fd, unsigned groups,
+	       int arg1, int arg2, int arg3);
    unsigned groups;
    int arg1;
    int arg2;
-   void *argp1;
+   int arg3;
 #if WITH_HELP
    const char *syntax;
 #endif
@@ -193,17 +251,17 @@ struct addrdesc {
 
 #define XIO_WRITABLE(s) (((s)->common.flags+1)&2)
 #define XIO_READABLE(s) (((s)->common.flags+1)&1)
-#define XIO_RDSTREAM(s) (((s)->tag==XIO_TAG_DUAL)?&(s)->dual.stream[0]->stream:&(s)->stream)
-#define XIO_WRSTREAM(s) (((s)->tag==XIO_TAG_DUAL)?&(s)->dual.stream[1]->stream:&(s)->stream)
-#define XIO_GETRDFD(s) (((s)->tag==XIO_TAG_DUAL)?(s)->dual.stream[0]->stream.fd:(s)->stream.fd)
-#define XIO_GETWRFD(s) (((s)->tag==XIO_TAG_DUAL)?(s)->dual.stream[1]->stream.fd:((s)->stream.dtype==DATA_2PIPE)?(s)->stream.para.exec.fdout:((s)->stream.dtype==DATA_PIPE)?(s)->stream.para.bipipe.fdout:(s)->stream.fd)
+#define XIO_RDSTREAM(s) (((s)->tag==XIO_TAG_DUAL)?(s)->dual.stream[0]:&(s)->stream)
+#define XIO_WRSTREAM(s) (((s)->tag==XIO_TAG_DUAL)?(s)->dual.stream[1]:&(s)->stream)
+#define XIO_GETRDFD(s) (((s)->tag==XIO_TAG_DUAL)?(s)->dual.stream[0]->fd:(s)->stream.fd)
+#define XIO_GETWRFD(s) (((s)->tag==XIO_TAG_DUAL)?(s)->dual.stream[1]->fd:((s)->stream.dtype==DATA_2PIPE)?(s)->stream.para.exec.fdout:((s)->stream.dtype==DATA_PIPE)?(s)->stream.para.bipipe.fdout:(s)->stream.fd)
 #define XIO_EOF(s) (XIO_RDSTREAM(s)->eof && !XIO_RDSTREAM(s)->ignoreeof)
 
 typedef unsigned long flags_t;
 
 union integral {
    int            u_bool;
-   byte_t         u_byte;
+   uint8_t        u_byte;
    gid_t          u_gidt;
    int	          u_int;
    long           u_long;
@@ -219,11 +277,11 @@ union integral {
    unsigned int   u_uint;
    unsigned long  u_ulong;
    unsigned short u_ushort;
-   byte2_t        u_2bytes;
+   uint16_t       u_2bytes;
    void          *u_ptr;
    flags_t        u_flag;
    struct {
-      byte_t     *b_data;
+      uint8_t    *b_data;
       size_t      b_len;
    }              u_bin;
    struct timeval u_timeval;
@@ -270,15 +328,15 @@ extern xiofile_t *sock[XIO_MAXSOCK];
 				   not even by external changes correctable */
 
 extern int xioinitialize(void);
+extern int xio_forked_inchild(void);
 extern int xiosetopt(char what, const char *arg);
 extern int xioinqopt(char what, char *arg, size_t n);
 extern xiofile_t *xioopen(const char *args, int flags);
-extern int xioopensingle(char *addr, xiofile_t *fd, int xioflags);
+extern int xioopensingle(char *addr, struct single *xfd, int xioflags);
 extern int xioopenhelp(FILE *of, int level);
 
 /* must be outside function for use by childdied handler */
 extern xiofile_t *sock1, *sock2;
-extern int xioinitialize(void);
 extern pid_t diedunknown1;	/* child died before it is registered */
 extern pid_t diedunknown2;
 extern pid_t diedunknown3;
@@ -295,4 +353,6 @@ extern int xioshutdown(xiofile_t *sock, int how);
 extern int xioclose(xiofile_t *sock);
 extern void xioexit(void);
 
+extern int (*xiohook_newchild)(void);	/* xio calls this function from a new child process */
+
 #endif /* !defined(__xio_h_included) */
