@@ -1,4 +1,4 @@
-/* $Id: xioclose.c,v 1.23 2006/01/29 16:34:55 gerhard Exp $ */
+/* $Id: xioclose.c,v 1.25 2006/05/06 14:20:28 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -65,11 +65,11 @@ int xioclose1(struct single *pipe) {
       switch (pipe->howtoend) {
       case END_CLOSE: case END_CLOSE_KILL:
 	 if (Close(pipe->fd) < 0) {
-	 Warn2("close(%d): %s", pipe->fd, strerror(errno)); } break;
+	 Info2("close(%d): %s", pipe->fd, strerror(errno)); } break;
 #if WITH_SOCKET
       case END_SHUTDOWN: case END_SHUTDOWN_KILL:
 	 if (Shutdown(pipe->fd, 2) < 0) {
-	     Warn3("shutdown(%d, %d): %s", pipe->fd, 2, strerror(errno)); }
+	     Info3("shutdown(%d, %d): %s", pipe->fd, 2, strerror(errno)); }
          break;
 #endif /* WITH_SOCKET */
       case END_UNLINK: if (Unlink((const char *)pipe->name) < 0) {
@@ -107,8 +107,8 @@ int xioclose(xiofile_t *file) {
    }
 
    if (file->tag == XIO_TAG_DUAL) {
-      result  = xioclose1(&file->dual.stream[0]->stream);
-      result |= xioclose1(&file->dual.stream[1]->stream);
+      result  = xioclose1(file->dual.stream[0]);
+      result |= xioclose1(file->dual.stream[1]);
       file->tag = XIO_TAG_INVALID;
    } else {
       result = xioclose1(&file->stream);
