@@ -1,5 +1,5 @@
-/* $Id: xioinitialize.c,v 1.13 2005/08/18 19:55:36 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: xioinitialize.c,v 1.16 2006/06/30 20:35:21 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for the initialize function */
@@ -9,13 +9,15 @@
 #include "xioopen.h"
 #include "xiolockfile.h"
 
+#include "xio-openssl.h"	/* xio_reset_fips_mode() */
 
 static int xioinitialized;
 xiofile_t *sock[XIO_MAXSOCK];
-int (*xiohook_newchild)(pid_t);	/* xio calls this function from a new child
+int (*xiohook_newchild)(void);	/* xio calls this function from a new child
 				   process */
 
 
+/* returns 0 on success or != if an error occurred */
 int xioinitialize(void) {
    if (xioinitialized)  return 0;
 
@@ -71,6 +73,31 @@ int xioinitialize(void) {
    assert(F_GETFD == F_SETFD-1);
    assert(F_GETFL == F_SETFL-1);
 
+   {
+      const char *default_ip;
+      default_ip = getenv("SOCAT_DEFAULT_LISTEN_IP");
+      if (default_ip != NULL) {
+	 switch (default_ip[0]) {
+	 case '4':
+	 case '6':
+	    xioopts.default_ip = default_ip[0]; break;
+	 }
+      }
+   }
+   {
+      const char *preferred_ip;
+      preferred_ip = getenv("SOCAT_PREFERRED_RESOLVE_IP");
+      if (preferred_ip != NULL) {
+	 switch (preferred_ip[0]) {
+	 case '4':
+	 case '6':
+	    xioopts.preferred_ip = preferred_ip[0]; break;
+	 default:
+	    xioopts.preferred_ip = '0'; break;
+	 }
+      }
+   }
+
    if (Atexit(xioexit) < 0) {
       Error("atexit(xioexit) failed");
       return -1;
@@ -95,3 +122,23 @@ void xiodroplocks(void) {
       }
    }
 }
+
+/* call this function immediately after fork() in child process */
+/* it performs some neccessary actions
+   returns 0 on success or != 0 if an error occurred */
+int xio_forked_inchild(void) {
+   int result = 0;
+   xiodroplocks();
+#if WITH_FIPS
+   if (xio_reset_fips_mode() != 0) {
+      result = 1;
+   }
+#endif /* WITH_FIPS */
+   /* some locks belong to parent process, so "drop" them now */
+   if (xiohook_newchild) {
+      if ((*xiohook_newchild)() != 0) {
+	 Exit(1);
+      }
+   }
+   return result;
+}
