@@ -1,5 +1,5 @@
-/* $Id: xiolayer.h,v 1.3 2001/11/04 17:19:21 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001 */
+/* $Id: xiolayer.h,v 1.5 2005/08/18 19:56:05 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xiolayer_h_included
@@ -8,6 +8,9 @@
 extern const struct optdesc opt_ignoreeof;
 extern const struct optdesc opt_cr;
 extern const struct optdesc opt_crnl;
+extern const struct optdesc opt_readbytes;
+extern const struct optdesc opt_lockfile;
+extern const struct optdesc opt_waitlock;
 extern const struct optdesc opt_forever;
 extern const struct optdesc opt_intervall;
 extern const struct optdesc opt_retry;
