@@ -1,5 +1,5 @@
-/* $Id: xioopts.h,v 1.54 2005/09/03 16:51:36 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: xioopts.h,v 1.64 2006/07/08 10:03:44 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xioopts_h_included
@@ -61,12 +61,15 @@ enum e_func {
    OFUNC_SEEK64,	/* lseek64(): arg1 is whence (SEEK_SET etc.) */
    OFUNC_FCNTL,		/* fcntl(, ): arg1 is cmd */
    OFUNC_IOCTL,		/* ioctl(): arg1 is request */
+   OFUNC_IOCTL_MASK_LONG,	/* arg1 is getrequest, arg2 is setrequest:
+			   ioctl(arg1, ); |= arg3; ioctl(arg2, ); */
    OFUNC_SOCKOPT,	/* setsockopt() */
    OFUNC_SOCKOPT_APPEND,/* getsockopt(), append data, setsockopt() */
    OFUNC_FLOCK,		/* flock() */
    OFUNC_TERMIO,	/* termio() ? */
    OFUNC_SPEC,		/* special, i.e. no generalizable function call */
    OFUNC_OFFSET,	/* put a value into xiofile struct; major is offset */
+   OFUNC_OFFSET_MASKS,	/* !!! */
    /*OFUNC_APPL,*/	/* special, i.e. application must know which f. */
    OFUNC_EXT,		/* with extended file descriptors only */
    OFUNC_TERMIOS_FLAG,	/* a flag in struct termios: major..tcflag, minor..bit
@@ -77,7 +80,8 @@ enum e_func {
    OFUNC_TERMIOS_CHAR,	/* a termios functional character: major..c_cc index */
    OFUNC_TERMIOS_SPEED,	/* termios c_ispeed etc on FreeBSD */
    OFUNC_TERMIOS_SPEC,	/* termios combined modes */
-   OFUNC_SIGNAL		/* a signal that should be passed to child process */
+   OFUNC_SIGNAL,	/* a signal that should be passed to child process */
+   OFUNC_RESOLVER	/* a bit position used on _res.options */
 } ;
 
 /* for simpler handling of option-to-connection-type relations we define
@@ -114,7 +118,7 @@ enum e_func {
 #define GROUP_FORK	0x00000800	/* communication with forked process */
 
 #define GROUP_LISTEN	0x00001000	/* socket in listening mode */
-#define GROUP_DEVICE	0x00002000
+/*	0x00002000 */
 #define GROUP_CHILD	0x00004000	/* autonom child process */
 #define GROUP_RETRY	0x00008000	/* when open/connect etc. fails */
 #define GROUP_TERMIOS	0x00010000
@@ -200,6 +204,7 @@ enum e_optcode {
    OPT_CLOCAL,		/* termios.c_cflag */
    OPT_CLOEXEC,
    OPT_CONNECT_TIMEOUT,	/* socket connect */
+   OPT_COOL_WRITE,
    OPT_CR,		/* customized */
 #ifdef CR0
    OPT_CR0,		/* termios.c_oflag */
@@ -237,6 +242,18 @@ enum e_optcode {
 #ifdef ECHOPRT
    OPT_ECHOPRT,		/* termios.c_lflag */
 #endif
+   OPT_EXT2_SECRM,
+   OPT_EXT2_UNRM,
+   OPT_EXT2_COMPR,
+   OPT_EXT2_SYNC,
+   OPT_EXT2_IMMUTABLE,
+   OPT_EXT2_APPEND,
+   OPT_EXT2_NODUMP,
+   OPT_EXT2_NOATIME,
+   OPT_EXT2_JOURNAL_DATA,
+   OPT_EXT2_NOTAIL,
+   OPT_EXT2_DIRSYNC,
+   OPT_EXT2_TOPDIR,
    OPT_FDIN,
    OPT_FDOUT,
 #ifdef FFDLY
@@ -280,6 +297,7 @@ enum e_optcode {
    OPT_INLCR,		/* termios.c_iflag */
    OPT_INPCK,		/* termios.c_iflag */
    OPT_INTERVALL,
+   OPT_IPV6_V6ONLY,
 #if 0	/* see Linux: man 7 netlink; probably not what we need yet */
    OPT_IO_SIOCGIFNAME,
 #endif
@@ -376,6 +394,7 @@ enum e_optcode {
    OPT_OPENSSL_CIPHERLIST,
    OPT_OPENSSL_DHPARAM,
    OPT_OPENSSL_EGD,
+   OPT_OPENSSL_FIPS,
    OPT_OPENSSL_KEY,
    OPT_OPENSSL_METHOD,
    OPT_OPENSSL_PSEUDO,
@@ -410,6 +429,7 @@ enum e_optcode {
 #if defined(O_NDELAY) && (!defined(O_NONBLOCK) || O_NDELAY != O_NONBLOCK)
    OPT_O_NDELAY,
 #endif
+   OPT_O_NOATIME,
    OPT_O_NOCTTY,
 #ifdef O_NOFOLLOW
    OPT_O_NOFOLLOW,
@@ -449,6 +469,7 @@ enum e_optcode {
    OPT_PIPES,
    /*OPT_PORT,*/
    OPT_PROMPT,		/* readline */
+   OPT_PROTOCOL_FAMILY,
    OPT_PROXYPORT,
    OPT_PROXY_AUTHORIZATION,
    OPT_PROXY_RESOLVE,
@@ -461,6 +482,15 @@ enum e_optcode {
    OPT_RANGE,		/* restrict client socket address */
    OPT_RAW,		/* termios */
    OPT_READBYTES,
+   OPT_RES_AAONLY,	/* resolver(3) */
+   OPT_RES_DEBUG,	/* resolver(3) */
+   OPT_RES_DEFNAMES,	/* resolver(3) */
+   OPT_RES_DNSRCH,	/* resolver(3) */
+   OPT_RES_IGNTC,	/* resolver(3) */
+   OPT_RES_PRIMARY,	/* resolver(3) */
+   OPT_RES_RECURSE,	/* resolver(3) */
+   OPT_RES_STAYOPEN,	/* resolver(3) */
+   OPT_RES_USEVC,	/* resolver(3) */
    OPT_RETRY,
    OPT_SANE,		/* termios */
    OPT_SEEK32_CUR,
@@ -597,6 +627,9 @@ enum e_optcode {
    OPT_TABDLY,		/* termios.c_oflag */
 #endif
    OPT_TCPWRAPPERS,	/* libwrap */
+   OPT_TCPWRAP_ETC,	/* libwrap */
+   OPT_TCPWRAP_HOSTS_ALLOW_TABLE,	/* libwrap */
+   OPT_TCPWRAP_HOSTS_DENY_TABLE,	/* libwrap */
    OPT_TCP_ABORT_THRESHOLD,	/* HP-UX */
    OPT_TCP_CONN_ABORT_THRESHOLD,	/* HP-UX */
 #ifdef TCP_CORK
@@ -754,7 +787,7 @@ struct optdesc {
    int  major;		/* major id for func: level (SOL_...) for setsockopt(),
 			   request for ioctl() */
    int  minor;	/* minor id for func: SO_..., IP_..., */
-   int  arg3;
+   long arg3;
 } ;
 
 extern bool xioopts_ignoregroups;
@@ -773,8 +806,10 @@ extern int retropt_string(struct opt *opts, int optcode, char **result);
 extern int retropt_timespec(struct opt *opts, int optcode, struct timespec *result);
 extern int retropt_bind(struct opt *opts,
 		 int af,
-		 const char *protoname,	/* for service resolution */
+		 int socktype,
+		 int ipproto,
 		 struct sockaddr *sa,
+		 socklen_t *salen,
 		 int feats	/* TCP etc: 1..address allowed,
 				   3..address and port allowed */
 		 );
@@ -789,7 +824,7 @@ extern int applyopts_single(struct single *fd, struct opt *opts, enum e_phase ph
 extern int applyopts_offset(struct single *xfd, struct opt *opts);
 extern int applyopts_signal(struct single *xfd, struct opt *opts);
 extern int _xio_openlate(struct single *fd, struct opt *opts);
-extern int parseopts(char *a, unsigned int groups, struct opt **opts);
+extern int parseopts(const char **a, unsigned int groups, struct opt **opts);
 extern struct opt *copyopts(const struct opt *opts, unsigned int groups);
 extern struct opt *moveopts(struct opt *opts, unsigned int groups);
 extern int leftopts(const struct opt *opts);
