@@ -1,5 +1,5 @@
-/* $Id: xiosigchld.c,v 1.18 2005/03/13 12:20:43 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: xiosigchld.c,v 1.20 2006/07/02 07:11:31 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the source of the extended child signal handler */
@@ -20,8 +20,8 @@ int xiosetsigchild(xiofile_t *xfd, int (*callback)(struct single *)) {
    if (xfd->tag != XIO_TAG_DUAL) {
       xfd->stream.sigchild = callback;
    } else {
-      xfd->dual.stream[0]->stream.sigchild = callback;
-      xfd->dual.stream[1]->stream.sigchild = callback;
+      xfd->dual.stream[0]->sigchild = callback;
+      xfd->dual.stream[1]->sigchild = callback;
    }
    return 0;
 }
@@ -51,10 +51,10 @@ static int xio_checkchild(xiofile_t *socket, int socknum, pid_t deadchild) {
 	    return 1;
 	 }
       } else {
-	 if (retval = xio_checkchild(socket->dual.stream[0], socknum, deadchild))
+	 if (retval = xio_checkchild((xiofile_t *)socket->dual.stream[0], socknum, deadchild))
 	    return retval;
 	 else
-	    return xio_checkchild(socket->dual.stream[1], socknum, deadchild);
+	    return xio_checkchild((xiofile_t *)socket->dual.stream[1], socknum, deadchild);
       }
    }
    return 0;
@@ -70,7 +70,7 @@ static int xio_checkchild(xiofile_t *socket, int socknum, pid_t deadchild) {
 void childdied(int signum) {
    pid_t pid;
    int _errno;
-   int status;
+   int status = 0;
    bool wassig = false;
    int i;
 
@@ -81,10 +81,15 @@ void childdied(int signum) {
    Info1("childdied(signum=%d)", signum);
    do {
       pid = Waitpid(-1, &status, WNOHANG);
-      if (pid == 0 || (pid < 0 && errno == ECHILD)) {
-	 if (!wassig) {
-	    Warn("waitpid(-1, {}, WNOHANG): no child process");
-	 }
+      if (pid == 0) {
+	 Msg(wassig?E_INFO:E_WARN,
+	     "waitpid(-1, {}, WNOHANG): no child has exited");
+	 Info("childdied() finished");
+	 errno = _errno;
+	 return;
+      } else if (pid < 0 && errno == ECHILD) {
+	 Msg1(wassig?E_INFO:E_WARN,
+	      "waitpid(-1, {}, WNOHANG): %s", strerror(errno));
 	 Info("childdied() finished");
 	 errno = _errno;
 	 return;
