@@ -1,5 +1,5 @@
-/* $Id: xiowrite.c,v 1.20 2005/03/18 20:26:13 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: xiowrite.c,v 1.25 2006/07/08 10:03:47 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the source of the extended write function */
@@ -12,9 +12,12 @@
 #include "xio-openssl.h"
 
 
+/* ...
+   on return value < 0: errno reflects the value from write() */
 ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
    ssize_t writt;
    struct single *pipe;
+   int _errno;
 
    if (file->tag == XIO_TAG_INVALID) {
       Error("xiowrite(): invalid file descriptor");
@@ -23,7 +26,7 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
    }
 
    if (file->tag == XIO_TAG_DUAL) {
-      pipe = &file->dual.stream[1]->stream;
+      pipe = file->dual.stream[1];
       if (pipe->tag == XIO_TAG_INVALID) {
 	 Error("xiowrite(): invalid file descriptor");
 	 errno = EINVAL;
@@ -46,17 +49,21 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
 	 writt = Write(pipe->fd, buff, bytes);
       } while (writt < 0 && errno == EINTR);
       if (writt < 0) {
-	 switch (errno) {
-#if 1
-	 case EPIPE: case ECONNRESET:
-	    Warn4("write(%d, %p, "F_Zu"): %s",
-		  pipe->fd, buff, bytes, strerror(errno));
-	    break;
-#endif
+	 _errno = errno;
+	 switch (_errno) {
+	 case EPIPE:
+	 case ECONNRESET:
+	    if (pipe->cool_write) {
+	       Notice4("write(%d, %p, "F_Zu"): %s",
+		       pipe->fd, buff, bytes, strerror(_errno));
+	       break;
+	    }
+	    /*PASSTRHOUGH*/
 	 default:
 	    Error4("write(%d, %p, "F_Zu"): %s",
-		   pipe->fd, buff, bytes, strerror(errno));
+		   pipe->fd, buff, bytes, strerror(_errno));
 	 }
+	 errno = _errno;
 	 return -1;
       }
       if ((size_t)writt < bytes) {
@@ -64,7 +71,7 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
 	       writt, bytes);
       }
 #if WITH_SOCKET
-   } else if (pipe->dtype == DATA_RECV) {
+   } else if (pipe->dtype == DATA_RECVFROM || pipe->dtype == DATA_RECVFROM_SKIPIP) {
       /*union {
 	 char space[sizeof(struct sockaddr_un)];
 	 struct sockaddr sa;
@@ -77,11 +84,13 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
       } while (writt < 0 && errno == EINTR);
       if (writt < 0) {
 	 char infobuff[256];
+	 _errno = errno;
 	 Error6("sendto(%d, %p, "F_Zu", 0, %s, "F_socklen"): %s",
 		pipe->fd, buff, bytes, 
 		 sockaddr_info(&pipe->peersa.soa,
 			       infobuff, sizeof(infobuff)),
-		 pipe->salen, strerror(errno));
+		 pipe->salen, strerror(_errno));
+	 errno = _errno;
 	 return -1;
       }
       if ((size_t)writt < bytes) {
@@ -93,14 +102,24 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
 	       pipe->salen, writt, bytes);
       } else {
       }
+      {
+	 char infobuff[256];
+	 union sockaddr_union us;
+	 socklen_t uslen = sizeof(us);
+	 Getsockname(pipe->fd, &us.soa, &uslen);
+	 Notice1("local address: %s",
+		 sockaddr_info(&us.soa, infobuff, sizeof(infobuff)));
+      }
 #endif /* WITH_SOCKET */
    } else if (pipe->dtype == DATA_PIPE) {
       do {
 	 writt = Write(pipe->para.bipipe.fdout, buff, bytes);
       } while (writt < 0 && errno == EINTR);
+      _errno = errno;
       if (writt < 0) {
 	 Error4("write(%d, %p, "F_Zu"): %s",
-		pipe->para.bipipe.fdout, buff, bytes, strerror(errno));
+		pipe->para.bipipe.fdout, buff, bytes, strerror(_errno));
+	 errno = _errno;
 	 return -1;
       }
       if ((size_t)writt < bytes) {
@@ -111,9 +130,11 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
       do {
 	 writt = Write(pipe->para.exec.fdout, buff, bytes);
       } while (writt < 0 && errno == EINTR);
+      _errno = errno;
       if (writt < 0) {
 	 Error4("write(%d, %p, "F_Zu"): %s",
-		pipe->para.exec.fdout, buff, bytes, strerror(errno));
+		pipe->para.exec.fdout, buff, bytes, strerror(_errno));
+	 errno = _errno;
 	 return -1;
       }
       if ((size_t)writt < bytes) {
@@ -122,11 +143,12 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
       }
 #if WITH_OPENSSL
    } else if (pipe->dtype == DATA_OPENSSL) {
-      /* this function prints its error messages */
+      /* this function prints its own error messages */
       return xiowrite_openssl(pipe, buff, bytes);
 #endif /* WITH_OPENSSL */
    } else {
       Error1("xiowrite(): bad data type specification %d", pipe->dtype);
+      errno = EINVAL;
       return -1;
    }
    return writt;
